/**
 * Class which wraps ResultSet in order to provide a close method
 * which also closes the statement.
 * we replace the close method only and make it close the statement
 * as well, this is only neccessary due to needing the statement still
 * open for Connector/J resultsets to be valid as opposed to 
 * org.gjt.mm.mysql ones.
 * Most just genreated via perl script from java.sql.ResultSet
 */

package org.ucam.srcf.assassins;

import org.ucam.srcf.assassins.exceptions.AssassinSQLException;
import java.math.BigDecimal;
import java.net.URL;
import java.util.Calendar;
import java.util.Map;
import java.io.InputStream;
import java.io.Reader;
import java.sql.*;

public class AResultSet implements ResultSet {
	private ResultSet encapsulated=null;
	private Statement stat=null;

	AResultSet(ResultSet r, Statement s) throws AssassinSQLException {
		this.encapsulated=(ResultSet) r;
		this.stat=s;
	}
	
	public void close() throws SQLException {
		/* It is also noted that Connector/J differs from 
		 * org.gjt.mm.mysql in that if a result returns no
		 * result sets then we can't retreive them under 
		 * Connector/J. We handle this by testing for nullness
		 * here instead to maintain backwards compat.
		 */
		/* Closes the statment which generated this, which in turn
		 * cloese the encapsualted ResultSet */
		if(stat!=null)
			stat.close();
		stat=null;
	}

	/* ***********************************************************
	 * Written in from compile errors
	 * ***********************************************************/
	public byte[] getBytes(String val)
		throws SQLException
	{
		return encapsulated.getBytes(val);
	}

	public byte[] getBytes(int val)
		throws SQLException
	{
		return encapsulated.getBytes(val);
	}

	public void updateBytes(int val1, byte[] val2)
		throws SQLException
	{
		encapsulated.updateBytes(val1, val2);
	}
	
	public void updateBytes(String val1, byte[] val2)
		throws SQLException
	{
		encapsulated.updateBytes(val1, val2);
	}

	public void updateCharacterStream(String var1, Reader var2, int var3)
		throws SQLException
	{
		encapsulated.updateCharacterStream(var1,var2,var3);
	}
	
	/* ***********************************************************
	 * Below here is completely just rewritten via perl
	 * ***********************************************************/
	
	public boolean next() throws SQLException
	{
		return encapsulated.next();
	}

	public boolean wasNull() throws SQLException
	{
		return encapsulated.wasNull();
	}

	public String getString(int columnIndex) throws SQLException
	{
		return encapsulated.getString(columnIndex);
	}

	public boolean getBoolean(int columnIndex) throws SQLException
	{
		return encapsulated.getBoolean(columnIndex);
	}

	public byte getByte(int columnIndex) throws SQLException
	{
		return encapsulated.getByte(columnIndex);
	}

	public short getShort(int columnIndex) throws SQLException
	{
		return encapsulated.getShort(columnIndex);
	}

	public int getInt(int columnIndex) throws SQLException
	{
		return encapsulated.getInt(columnIndex);
	}

	public long getLong(int columnIndex) throws SQLException
	{
		return encapsulated.getLong(columnIndex);
	}

	public float getFloat(int columnIndex) throws SQLException
	{
		return encapsulated.getFloat(columnIndex);
	}

	public double getDouble(int columnIndex) throws SQLException
	{
		return encapsulated.getDouble(columnIndex);
	}

	public BigDecimal getBigDecimal(int columnIndex, int scale)
		throws SQLException
	{
		return encapsulated.getBigDecimal(columnIndex, scale);
	}

	public Date getDate(int columnIndex) throws SQLException
	{
		return encapsulated.getDate(columnIndex);
	}

	public Time getTime(int columnIndex) throws SQLException
	{
		return encapsulated.getTime(columnIndex);
	}

	public Timestamp getTimestamp(int columnIndex) throws SQLException
	{
		return encapsulated.getTimestamp(columnIndex);
	}

	public InputStream getAsciiStream(int columnIndex) throws SQLException
	{
		return encapsulated.getAsciiStream(columnIndex);
	}

	public InputStream getUnicodeStream(int columnIndex) throws SQLException
	{
		return encapsulated.getUnicodeStream(columnIndex);
	}

	public InputStream getBinaryStream(int columnIndex) throws SQLException
	{
		return encapsulated.getBinaryStream(columnIndex);
	}

	public String getString(String columnName) throws SQLException
	{
		return encapsulated.getString(columnName);
	}

	public boolean getBoolean(String columnName) throws SQLException
	{
		return encapsulated.getBoolean(columnName);
	}

	public byte getByte(String columnName) throws SQLException
	{
		return encapsulated.getByte(columnName);
	}

	public short getShort(String columnName) throws SQLException
	{
		return encapsulated.getShort(columnName);
	}

	public int getInt(String columnName) throws SQLException
	{
		return encapsulated.getInt(columnName);
	}

	public long getLong(String columnName) throws SQLException
	{
		return encapsulated.getLong(columnName);
	}

	public float getFloat(String columnName) throws SQLException
	{
		return encapsulated.getFloat(columnName);
	}

	public double getDouble(String columnName) throws SQLException
	{
		return encapsulated.getDouble(columnName);
	}

	public BigDecimal getBigDecimal(String columnName, int scale)
		throws SQLException
	{
		return encapsulated.getBigDecimal(columnName, scale);
	}

	public Date getDate(String columnName) throws SQLException
	{
		return encapsulated.getDate(columnName);
	}

	public Time getTime(String columnName) throws SQLException
	{
		return encapsulated.getTime(columnName);
	}

	public Timestamp getTimestamp(String columnName) throws SQLException
	{
		return encapsulated.getTimestamp(columnName);
	}

	public InputStream getAsciiStream(String columnName) throws SQLException
	{
		return encapsulated.getAsciiStream(columnName);
	}

	public InputStream getUnicodeStream(String columnName) throws SQLException
	{
		return encapsulated.getUnicodeStream(columnName);
	}

	public InputStream getBinaryStream(String columnName) throws SQLException
	{
		return encapsulated.getBinaryStream(columnName);
	}

	public SQLWarning getWarnings() throws SQLException
	{
		return encapsulated.getWarnings();
	}

	public void clearWarnings() throws SQLException
	{
		encapsulated.clearWarnings();
	}

	public String getCursorName() throws SQLException
	{
		return encapsulated.getCursorName();
	}

	public ResultSetMetaData getMetaData() throws SQLException
	{
		return encapsulated.getMetaData();
	}

	public Object getObject(int columnIndex) throws SQLException
	{
		return encapsulated.getObject(columnIndex);
	}

	public Object getObject(String columnName) throws SQLException
	{
		return encapsulated.getObject(columnName);
	}

	public int findColumn(String columnName) throws SQLException
	{
		return encapsulated.findColumn(columnName);
	}

	public Reader getCharacterStream(int columnIndex) throws SQLException
	{
		return encapsulated.getCharacterStream(columnIndex);
	}

	public Reader getCharacterStream(String columnName) throws SQLException
	{
		return encapsulated.getCharacterStream(columnName);
	}

	public BigDecimal getBigDecimal(int columnIndex) throws SQLException
	{
		return encapsulated.getBigDecimal(columnIndex);
	}

	public BigDecimal getBigDecimal(String columnName) throws SQLException
	{
		return encapsulated.getBigDecimal(columnName);
	}

	public boolean isBeforeFirst() throws SQLException
	{
		return encapsulated.isBeforeFirst();
	}

	public boolean isAfterLast() throws SQLException
	{
		return encapsulated.isAfterLast();
	}

	public boolean isFirst() throws SQLException
	{
		return encapsulated.isFirst();
	}

	public boolean isLast() throws SQLException
	{
		return encapsulated.isLast();
	}

	public void beforeFirst() throws SQLException
	{
		encapsulated.beforeFirst();
	}

	public void afterLast() throws SQLException
	{
		encapsulated.afterLast();
	}

	public boolean first() throws SQLException
	{
		return encapsulated.first();
	}

	public boolean last() throws SQLException
	{
		return encapsulated.last();
	}

	public int getRow() throws SQLException
	{
		return encapsulated.getRow();
	}

	public boolean absolute(int row) throws SQLException
	{
		return encapsulated.absolute(row);
	}

	public boolean relative(int rows) throws SQLException
	{
		return encapsulated.relative(rows);
	}

	public boolean previous() throws SQLException
	{
		return encapsulated.previous();
	}

	public void setFetchDirection(int direction) throws SQLException
	{
		encapsulated.setFetchDirection(direction);
	}

	public int getFetchDirection() throws SQLException
	{
		return encapsulated.getFetchDirection();
	}

	public void setFetchSize(int rows) throws SQLException
	{
		encapsulated.setFetchSize(rows);
	}

	public int getFetchSize() throws SQLException
	{
		return encapsulated.getFetchSize();
	}

	public int getType() throws SQLException
	{
		return encapsulated.getType();
	}

	public int getConcurrency() throws SQLException
	{
		return encapsulated.getConcurrency();
	}

	public boolean rowUpdated() throws SQLException
	{
		return encapsulated.rowUpdated();
	}

	public boolean rowInserted() throws SQLException
	{
		return encapsulated.rowInserted();
	}

	public boolean rowDeleted() throws SQLException
	{
		return encapsulated.rowDeleted();
	}

	public void updateNull(int columnIndex) throws SQLException
	{
		encapsulated.updateNull(columnIndex);
	}

	public void updateBoolean(int columnIndex, boolean x) throws SQLException
	{
		encapsulated.updateBoolean(columnIndex, x);
	}

	public void updateByte(int columnIndex, byte x) throws SQLException
	{
		encapsulated.updateByte(columnIndex, x);
	}

	public void updateShort(int columnIndex, short x) throws SQLException
	{
		encapsulated.updateShort(columnIndex, x);
	}

	public void updateInt(int columnIndex, int x) throws SQLException
	{
		encapsulated.updateInt(columnIndex, x);
	}

	public void updateLong(int columnIndex, long x) throws SQLException
	{
		encapsulated.updateLong(columnIndex, x);
	}

	public void updateFloat(int columnIndex, float x) throws SQLException
	{
		encapsulated.updateFloat(columnIndex, x);
	}

	public void updateDouble(int columnIndex, double x) throws SQLException
	{
		encapsulated.updateDouble(columnIndex, x);
	}

	public void updateBigDecimal(int columnIndex, BigDecimal x)
		throws SQLException
	{
		encapsulated.updateBigDecimal(columnIndex, x);
	}

	public void updateString(int columnIndex, String x) throws SQLException
	{
		encapsulated.updateString(columnIndex, x);
	}

	public void updateDate(int columnIndex, Date x) throws SQLException
	{
		encapsulated.updateDate(columnIndex, x);
	}

	public void updateTime(int columnIndex, Time x) throws SQLException
	{
		encapsulated.updateTime(columnIndex, x);
	}

	public void updateTimestamp(int columnIndex, Timestamp x)
		throws SQLException
	{
		encapsulated.updateTimestamp(columnIndex, x);
	}

	public void updateAsciiStream(int columnIndex, InputStream x, int length)
		throws SQLException
	{
		encapsulated.updateAsciiStream(columnIndex, x, length);
	}

	public void updateBinaryStream(int columnIndex, InputStream x, int length)
		throws SQLException
	{
		encapsulated.updateBinaryStream(columnIndex, x, length);
	}

	public void updateCharacterStream(int columnIndex, Reader x, int length)
		throws SQLException
	{
		encapsulated.updateCharacterStream(columnIndex, x, length);
	}

	public void updateObject(int columnIndex, Object x, int scale)
		throws SQLException
	{
		encapsulated.updateObject(columnIndex, x, scale);
	}

	public void updateObject(int columnIndex, Object x) throws SQLException
	{
		encapsulated.updateObject(columnIndex, x);
	}

	public void updateNull(String columnName) throws SQLException
	{
		encapsulated.updateNull(columnName);
	}

	public void updateBoolean(String columnName, boolean x) throws SQLException
	{
		encapsulated.updateBoolean(columnName, x);
	}

	public void updateByte(String columnName, byte x) throws SQLException
	{
		encapsulated.updateByte(columnName, x);
	}

	public void updateShort(String columnName, short x) throws SQLException
	{
		encapsulated.updateShort(columnName, x);
	}

	public void updateInt(String columnName, int x) throws SQLException
	{
		encapsulated.updateInt(columnName, x);
	}

	public void updateLong(String columnName, long x) throws SQLException
	{
		encapsulated.updateLong(columnName, x);
	}

	public void updateFloat(String columnName, float x) throws SQLException
	{
		encapsulated.updateFloat(columnName, x);
	}

	public void updateDouble(String columnName, double x) throws SQLException
	{
		encapsulated.updateDouble(columnName, x);
	}

	public void updateBigDecimal(String columnName, BigDecimal x)
		throws SQLException
	{
		encapsulated.updateBigDecimal(columnName, x);
	}

	public void updateString(String columnName, String x) throws SQLException
	{
		encapsulated.updateString(columnName, x);
	}

	public void updateDate(String columnName, Date x) throws SQLException
	{
		encapsulated.updateDate(columnName, x);
	}

	public void updateTime(String columnName, Time x) throws SQLException
	{
		encapsulated.updateTime(columnName, x);
	}

	public void updateTimestamp(String columnName, Timestamp x)
		throws SQLException
	{
		encapsulated.updateTimestamp(columnName, x);
	}

	public void updateAsciiStream(String columnName, InputStream x, int length)
		throws SQLException
	{
		encapsulated.updateAsciiStream(columnName, x, length);
	}

	public void updateBinaryStream(String columnName, InputStream x, int length)
		throws SQLException
	{
		encapsulated.updateBinaryStream(columnName, x, length);
	}

	public void updateObject(String columnName, Object x, int scale)
		throws SQLException
	{
		encapsulated.updateObject(columnName, x, scale);
	}

	public void updateObject(String columnName, Object x) throws SQLException
	{
		encapsulated.updateObject(columnName, x);
	}

	public void insertRow() throws SQLException
	{
		encapsulated.insertRow();
	}

	public void updateRow() throws SQLException
	{
		encapsulated.updateRow();
	}

	public void deleteRow() throws SQLException
	{
		encapsulated.deleteRow();
	}

	public void refreshRow() throws SQLException
	{
		encapsulated.refreshRow();
	}

	public void cancelRowUpdates() throws SQLException
	{
		encapsulated.cancelRowUpdates();
	}

	public void moveToInsertRow() throws SQLException
	{
		encapsulated.moveToInsertRow();
	}

	public void moveToCurrentRow() throws SQLException
	{
		encapsulated.moveToCurrentRow();
	}

	public Statement getStatement() throws SQLException
	{
		return encapsulated.getStatement();
	}

	public Object getObject(int i, Map map) throws SQLException
	{
		return encapsulated.getObject(i, map);
	}

	public Ref getRef(int i) throws SQLException
	{
		return encapsulated.getRef(i);
	}

	public Blob getBlob(int i) throws SQLException
	{
		return encapsulated.getBlob(i);
	}

	public Clob getClob(int i) throws SQLException
	{
		return encapsulated.getClob(i);
	}

	public Array getArray(int i) throws SQLException
	{
		return encapsulated.getArray(i);
	}

	public Object getObject(String colName, Map map) throws SQLException
	{
		return encapsulated.getObject(colName, map);
	}

	public Ref getRef(String colName) throws SQLException
	{
		return encapsulated.getRef(colName);
	}

	public Blob getBlob(String colName) throws SQLException
	{
		return encapsulated.getBlob(colName);
	}

	public Clob getClob(String colName) throws SQLException
	{
		return encapsulated.getClob(colName);
	}

	public Array getArray(String colName) throws SQLException
	{
		return encapsulated.getArray(colName);
	}

	public Date getDate(int columnIndex, Calendar cal) throws SQLException
	{
		return encapsulated.getDate(columnIndex, cal);
	}

	public Date getDate(String columnName, Calendar cal) throws SQLException
	{
		return encapsulated.getDate(columnName, cal);
	}

	public Time getTime(int columnIndex, Calendar cal) throws SQLException
	{
		return encapsulated.getTime(columnIndex, cal);
	}

	public Time getTime(String columnName, Calendar cal) throws SQLException
	{
		return encapsulated.getTime(columnName, cal);
	}

	public Timestamp getTimestamp(int columnIndex, Calendar cal)
		throws SQLException
	{
		return encapsulated.getTimestamp(columnIndex, cal);
	}

	public Timestamp getTimestamp(String columnName, Calendar cal)
		throws SQLException
	{
		return encapsulated.getTimestamp(columnName, cal);
	}

	public URL getURL(int columnIndex) throws SQLException
	{
		return encapsulated.getURL(columnIndex);
	}

	public URL getURL(String columnName) throws SQLException
	{
		return encapsulated.getURL(columnName);
	}

	public void updateRef(int columnIndex, Ref x) throws SQLException
	{
		encapsulated.updateRef(columnIndex, x);
	}

	public void updateRef(String columnName, Ref x) throws SQLException
	{
		encapsulated.updateRef(columnName, x);
	}

	public void updateBlob(int columnIndex, Blob x) throws SQLException
	{
		encapsulated.updateBlob(columnIndex, x);
	}

	public void updateBlob(String columnName, Blob x) throws SQLException
	{
		encapsulated.updateBlob(columnName, x);
	}

	public void updateClob(int columnIndex, Clob x) throws SQLException
	{
		encapsulated.updateClob(columnIndex, x);
	}

	public void updateClob(String columnName, Clob x) throws SQLException
	{
		encapsulated.updateClob(columnName, x);
	}

	public void updateArray(int columnIndex, Array x) throws SQLException
	{
		encapsulated.updateArray(columnIndex, x);
	}

	public void updateArray(String columnName, Array x) throws SQLException
	{
		encapsulated.updateArray(columnName, x);
	}
}

