/* 
 * CUAG AutoUmpire System
 * 
 * Copyright (C) 2003/2004 Matthew Johnson, Adam Biltcliffe,
 * Michael Cripps, Martin O'Leary, Edward Allcutt, Jonathan Hogg 
 * and James Osborn
 *
 * This program is free software; you can redistribute it and/or 
 * modify it under the terms of the GNU General Public License 
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version. 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details. 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * To Contact the authors, please email assassins@srcf.ucam.org
 *
 */

package org.ucam.srcf.assassins;

import java.util.Vector;
import java.util.Map;
import java.util.HashMap;

import java.sql.ResultSet;
import java.sql.SQLException;

import org.ucam.srcf.assassins.exceptions.AssassinSQLException;

import org.ucam.srcf.assassins.xml.Config;

/**
 * Class to provide a safe and secure authentication mechanism.
 * Implemented using cookies and password.
 */
public class Authentication {
	/** The one and only Authentication object */
	public static Authentication currentAuth = new Authentication();
	
	/** Email address of currently authenticated user -- ???? email?! its an int! (mjj29)*/
	private int userid=-1;
	/** database ID of this session */
	private int sid=-1;
	/** Privledge level we're logged in at */
	private AuthLevel priv_level=AuthLevel.PUBLIC;
	/** session id cookie contents */
	private String session_id="";
	/** PlayerViewer object for currently logged in player */
	private PlayerViewer loggedInPlayer=null;

	/** length of time for sessions to last for (seconds) */
	private static int sessionLength=600;
	/** email address of root user */
	private static String rootAddress="";

	/** Cache of session data */
	private Map sess_other_data=new HashMap();

	/**
	 * Constructor.
	 * This is private to enforce only one person can be Authenticated at once.
	 */
	private Authentication() {
	}

	/* read config when its convinient load */
	public static void readConfigValues()
	{
		if(Debug.debug) Debug.print(Authentication.class,"Reading config values");
		rootAddress=Config.getConfig("vanilla", "root-address");
		sessionLength=Config.getConfigInt("vanilla", "session-length");
      if (sessionLength == 0) sessionLength = 3600; // one hour
	}

	/**
	 * Cleans out auth table of db.
	 * Remvoes any old entries
	 */
	public void authClean() throws AssassinSQLException {
		if(Debug.debug) Debug.print(this,"Cleaning out expired session cookies");
		try {
         String query="SELECT sid FROM " + DatabaseBackend.getPrefix() + "_active_sessions WHERE expires < NOW()";
			ResultSet r = DatabaseBackend.query(query);
         while (r.next()) {
            int sid = r.getInt("sid");
            query="DELETE FROM " + DatabaseBackend.getPrefix() + 
               "_active_sessions WHERE sid = '"+sid+"'";
            DatabaseBackend.query(query).close();
            query="DELETE FROM " + DatabaseBackend.getPrefix() + 
               "_session_data WHERE sid = '"+sid+"'";
            DatabaseBackend.query(query).close();
            query="DELETE FROM " + DatabaseBackend.getPrefix() + 
               "_session_other_data WHERE sid = '"+sid+"'";
            DatabaseBackend.query(query).close();
         }
         r.close();
		} catch (SQLException e) {}
	}

	private void setAuthPublic() {
		userid = -1;
		priv_level = AuthLevel.PUBLIC;
		loggedInPlayer = null;
		session_id="";
		sess_other_data=new HashMap();
	}

	/**
	 * Check if a username/passowrd combination is valid.
	 * If so start a session for them, and generate and store a session_id
	 *
	 * @return The auth level of the user. PUBLIC if auth failed.
	 */
	public AuthLevel validateUser(String email, String password) throws AssassinSQLException {
		if(Debug.debug) {
			Debug.print(this, "Attempting to validate "+email+"/"+password);
			Debug.print(this, "Root address is "+rootAddress);
		}
		if (	null == email		||		null == password || 
				email.equals("")	||		password.equals("") ) {
			setAuthPublic();
			if(Debug.debug) Debug.print(this, "Failed to validate user "+email);
			return priv_level;
		}
		String query = "SELECT pid FROM "+DatabaseBackend.getPrefix()+"_players WHERE email='"+Tools.escape(email)+"' AND password='"+Tools.escape(password)+"'";
		try {
			ResultSet rs=DatabaseBackend.query(query);
			if(!rs.next()) {
				rs.close();
				setAuthPublic();
				if(Debug.debug) Debug.print(this, "Failed to validate user "+email);
				return priv_level;
			}
			userid=Integer.parseInt(rs.getString("pid"));
			rs.close();
			if(rootAddress.equals("")) {
				//FIXME: Do something sensible in here to complain no umpire defined?
				if(Debug.debug) Debug.print(this, "Failed to validate user "+email+", no umpire defined.");
				priv_level=AuthLevel.PUBLIC;
				return priv_level;
			} else if(rootAddress.equals(email)) {
				priv_level=AuthLevel.UMPIRE;
			} else {
				priv_level=AuthLevel.PLAYER;
			}
			session_id=generateSessionID(email);
			query = "INSERT INTO "+DatabaseBackend.getPrefix()+"_active_sessions (pid, session_id, expires, auth_level) VALUES ('"+userid+"', '"+session_id+"', DATE_ADD(NOW(), INTERVAL "+sessionLength+" SECOND),"+priv_level.getInt()+")";
			(DatabaseBackend.query(query)).close();
         query = "SELECT LAST_INSERT_ID()";
			rs = DatabaseBackend.query(query);
			rs.next();
         sid = rs.getInt(1);
         rs.close();
		} catch (Exception e) {
			setAuthPublic();
			if(Debug.debug) { 
            Debug.print(this, "Failed to validate user "+email);
            Debug.print(this, "There was an Exception: "+e);
         }
			return priv_level;
		}
		if(Debug.debug) Debug.print(this, "Validated "+email+" as "+ priv_level);
		readSessData();
		return priv_level;
	}

	/** 
	 * Generate a unique and unguessable session key.
	 * We could do this by taking an md5sum of a value based on the current
	 * sustem time and the email address. However MessageDigest is just
	 * messy.
	 * FIXME: Do this properly.
	 */
	private String generateSessionID(String email) {
		String sessID =  Tools.escape(email + System.currentTimeMillis());
		if(Debug.debug) Debug.print(this,"Generated Session ID: "+sessID);
		/* FIXME: Use a random number appended to email */
		return sessID;
	}
	
	/**
	 * Check if a session cookie is valid.
	 * If it is we update its expiration time, and make it the 
	 * current auth'd user. Returns the authlevel.
	 */
	public AuthLevel validateSession(String sessionID) throws AssassinSQLException {
		if(Debug.debug) Debug.print(this, "Attempting to validate Session: "+sessionID);
		if(sessionID == null) {
			setAuthPublic();
			if(Debug.debug) Debug.print(this, "Failed to validate Session: "+sessionID);
			return priv_level;
		}

		/* clean out expired sessions */
		authClean();
		/* See if we have a valid session */
		String query="SELECT * FROM " + DatabaseBackend.getPrefix() +"_active_sessions WHERE session_id='"+Tools.escape(sessionID)+"'";
		ResultSet rs = DatabaseBackend.query(query);
		try {
			if(!rs.next()) {
				/* Invalid session! We must be public. */
				try {
					rs.close();
				} catch(SQLException e) {}
				if(Debug.debug) Debug.print(this, "Failed to validate Session (not in db): "+sessionID);
				setAuthPublic();
				return priv_level;
			}
			/* move its expiry time ahead */
         sid = rs.getInt("sid");
			query="UPDATE "+DatabaseBackend.getPrefix()+"_active_sessions SET expires=DATE_ADD(NOW(), INTERVAL "+sessionLength+" SECOND) WHERE sid='"+sid+"'";
			try {
				(DatabaseBackend.query(query)).close();
			} catch(SQLException e) {}
			priv_level=AuthLevel.getLevel(rs.getInt("auth_level"));
			userid=Integer.parseInt(rs.getString("pid"));
			if(Debug.debug) Debug.print(this, "Validated Session "+sessionID+" at level of "+priv_level+" for userid "+userid);
			try {
				rs.close();
			} catch(SQLException e) {}
		} catch(SQLException e) {
			throw new AssassinSQLException(e);
		}
		readSessData();
		return priv_level;
	}

	/**
	 * Invalidate a user's session.
	 * Used to logout
	 * @return email of user logged out
	 */
	public String invalidateSession() {
		if(Debug.debug) Debug.print(this, "Invalidating current session, user was "+userid);
		try {
         String query="DELETE FROM "+DatabaseBackend.getPrefix()+"_active_sessions WHERE sid='"+sid+"'";
         (DatabaseBackend.query(query)).close();
         query="DELETE FROM "+DatabaseBackend.getPrefix()+"_session_data WHERE sid='"+sid+"'";
         (DatabaseBackend.query(query)).close();
         query="DELETE FROM "+DatabaseBackend.getPrefix()+"_session_other_data WHERE sid='"+sid+"'";
         (DatabaseBackend.query(query)).close();
		}
		catch (SQLException e) {}
		String ret="";
		try {
			ret=getEmail();
		} catch (AssassinSQLException ex) {}
		setAuthPublic();
		return ret;
	}

	/**
	 * Has the current user got at least better_than privledges?
	 */
	public boolean isAtLeast(AuthLevel better_than) {
		return (priv_level.compareTo(better_than) >= 0);
	}

	/**
	 * Returns the email for the currently logged in user.
	 */
	public String getEmail() throws AssassinSQLException {
		if(userid==-1) return "";		//no logged in player
		return getPlayerViewer().getEmail();
	}

	/**
	 * Returns an PlayerViewer for the currently logged in user.
	 */
	public PlayerViewer getPlayerViewer() {
		if(userid==-1) return null;	//no user logged in
		if(loggedInPlayer==null)
			loggedInPlayer = (PlayerViewer) (new Player(userid)).getViewer();
		return loggedInPlayer;
	}

	/**
	 * Returns the password the currently logged in user identifyed with.
	 * Should <b>only</b> ever be used for <b>debug</b>. Anything else 
	 * requireing knowledge fo the password should be written in this class.
	 * 
	 * @return the auth password if debugging is on, the empty stirng otherwise.
	 */
	public String getSecret() throws AssassinSQLException {
		if(userid==-1) return "";	//no user logged in
		if(Debug.debug) return getPlayerViewer().getPassword();
		return "";
	}

	/**
	 * Returns the session Id for the session cookie.
	 */
	public String getSessionID() {
		return session_id;
	}

	/**
	 * Returns the email address of the user with root access privs.
	 */
	public static String getRootUID() {
		return rootAddress;
	}

   /**
    * Returns all the questions saved in this session, for this plugin
    */
   public Vector getSessionQuestions(Plugin pl)
   {
      Vector v = new Vector();
      if (-1 == sid) return v;
      String query = "SELECT question FROM "+DatabaseBackend.getPrefix()+"_session_data WHERE sid = '"+sid+"' AND plugin='"+pl.getPluginName()+"' ORDER BY did";
      try {
         ResultSet r = DatabaseBackend.query(query);
         while (r.next()) {
            Question q = Question.deSerialise(r.getString("question"));
            v.add(q);
         }
         r.close();
      } catch (SQLException SQLe) {
         if(Debug.debug) Debug.print(Authentication.class,"SQL Error when getting session questions: "+SQLe);
      }
      return v;
   }

   /**
    * Returns all the questions saved in this session
    */
   public Vector getSessionQuestions() throws AssassinSQLException
   {
      Vector v = new Vector();
      if (-1 == sid) return v;
      String query = "SELECT question FROM "+DatabaseBackend.getPrefix()+"_session_data WHERE sid = '"+sid+"' ORDER BY did";
      try {
         ResultSet r = DatabaseBackend.query(query);
         while (r.next()) {
            Question q = Question.deSerialise(r.getString("question"));
            v.add(q);
         }
         r.close();
      } catch (SQLException SQLe) {
         if(Debug.debug) Debug.print(Authentication.class,"SQL Error when getting session questions: "+SQLe);
         if (SQLe instanceof AssassinSQLException) throw (AssassinSQLException) SQLe;
         else throw new AssassinSQLException();
      }
      return v;
   }
   /**
    * Clears the questions saved for the given plugin
    */
   public void clearSessionQuestions(Plugin p) throws AssassinSQLException
   {
      String query = "DELETE FROM "+DatabaseBackend.getPrefix()+"_session_data WHERE sid = '"+sid+"' AND plugin = '"+p.getPluginName()+"'";
      try {
         DatabaseBackend.query(query).close();
      } catch (SQLException SQLe) {
         if(Debug.debug) Debug.print(Authentication.class,"SQL Error when clearing session questions: "+SQLe);
         if (SQLe instanceof AssassinSQLException) throw (AssassinSQLException) SQLe;
         else throw new AssassinSQLException();
      }
   }

   /**
    * Clears the saved Questions
    */
   public void clearSessionQuestions() throws AssassinSQLException
   {
      String query = "DELETE FROM "+DatabaseBackend.getPrefix()+"_session_data WHERE sid = '"+sid+"'";
      try {
         DatabaseBackend.query(query).close();
      } catch (SQLException SQLe) {
         if(Debug.debug) Debug.print(Authentication.class,"SQL Error when clearing session questions: "+SQLe);
         if (SQLe instanceof AssassinSQLException) throw (AssassinSQLException) SQLe;
         else throw new AssassinSQLException();
      }
   }
   /**
    * Adds a saved Question
    */
   public void addSessionQuestion(Plugin pl, Question q) throws AssassinSQLException
   {
      String serial =Tools.escape(q.serialize());
      String query = "INSERT INTO "+DatabaseBackend.getPrefix()+"_session_data (sid, plugin, question) VALUES ('"+sid+"', '"+pl.getPluginName()+"', '"+serial+"')";
      try {
         DatabaseBackend.query(query).close();
      } catch (SQLException SQLe) {
         if(Debug.debug) Debug.print(Authentication.class,"SQL Error when adding session questions: "+SQLe);
         if (SQLe instanceof AssassinSQLException) throw (AssassinSQLException) SQLe;
         else throw new AssassinSQLException();
      }
   }

	/**
	 * Stores a piece of session data.
	 * We escape the data here.
	 */
	public void storeData(String field, String value) throws AssassinSQLException {
		/* First check if theres already an entry in there */
      if (Debug.debug) Debug.print(this, "Storing Data "+field+" => "+value);
		try {
			String query="SELECT * FROM "+DatabaseBackend.getPrefix()+"_session_other_data WHERE sid='"+sid+"' AND field='"+Tools.escape(field)+"'";
			ResultSet r=DatabaseBackend.query(query);
			if(!r.next()) {
				/* If no other entry, insert it */
				query="INSERT INTO "+DatabaseBackend.getPrefix()+"_session_other_data (sid, field, value) VALUES ('"+sid+"', '"+Tools.escape(field)+"', '"+Tools.escape(value)+"')";
			} else {
				/* update it */
				query="UPDATE "+DatabaseBackend.getPrefix()+"_session_other_data SET value='"+Tools.escape(value)+"' WHERE sid='"+sid+"' AND field='"+Tools.escape(field)+"'";
			}
			r.close();
			DatabaseBackend.query(query).close();
		} catch (SQLException SQLe) {
			if(Debug.debug) Debug.print(Authentication.class,"SQL Error when adding session data: "+SQLe);
         if (SQLe instanceof AssassinSQLException) throw (AssassinSQLException) SQLe;
         else throw new AssassinSQLException();
		}
		sess_other_data.put(field, value);
	}

   /**
    * Deletes a piece of session Data.
    */
   public void unsetData(String field) throws AssassinSQLException  {
      /* First make sure its there */
      if(sess_other_data.get(field) == null)
         return;
      /* Remove it from the cache */
      sess_other_data.remove(field);
      /* Remove it from the db */
      String query="DELETE FROM "+DatabaseBackend.getPrefix()+"_session_other_data WHERE sid='"+sid+"' AND field='"+Tools.escape(field)+"'";
      try {
         DatabaseBackend.query(query).close();
      } catch (SQLException SQLe) {
			if(Debug.debug) Debug.print(Authentication.class,"SQL Error when adding session data: "+SQLe);
         if (SQLe instanceof AssassinSQLException) throw (AssassinSQLException) SQLe;
         else throw new AssassinSQLException();
      }
   }

	/**
	 * Retrieve a piece of session data.
	 */
	public String getData(String field) {
		/* We've atually just cache all the data for this session */
		return (String) sess_other_data.get(field);
	}

	/**
	 * Read session other data into cache
	 */
	private void readSessData() throws AssassinSQLException  {
		String query="SELECT field,value FROM "+DatabaseBackend.getPrefix()+"_session_other_data WHERE sid='"+sid+"'";
		try {
			ResultSet r=DatabaseBackend.query(query);
			while(r.next()) {
				sess_other_data.put(r.getString("field"), r.getString("value"));
			}
			r.close();
		} catch ( SQLException SQLe) {
			if(Debug.debug) Debug.print(Authentication.class,"SQL Error when reading session data: "+SQLe);
         if (SQLe instanceof AssassinSQLException) throw (AssassinSQLException) SQLe;
         else throw new AssassinSQLException();
		}
	}
}

