/* 
 * CUAG AutoUmpire System
 * 
 * Copyright (C) 2003/2004 Matthew Johnson, Adam Biltcliffe,
 * Michael Cripps, Martin O'Leary, Edward Allcutt and James Osborn
 *
 * This program is free software; you can redistribute it and/or 
 * modify it under the terms of the GNU General Public License 
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version. 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details. 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * To Contact the authors, please email assassins@srcf.ucam.org
 *
 */

package org.ucam.srcf.assassins;

import java.io.IOException;
import java.text.ParseException;
import java.util.Collection;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Vector;

import cx.ath.matthew.cgi.CGI;
import cx.ath.matthew.cgi.CGIHeaderSentException;
import cx.ath.matthew.cgi.CGIInvalidContentFormatException;
import cx.ath.matthew.cgi.CheckBox;
import cx.ath.matthew.cgi.DisplayField;
import cx.ath.matthew.cgi.HiddenField;
import cx.ath.matthew.cgi.SubmitButton;


import org.ucam.srcf.assassins.exceptions.AssassinSQLException;
import org.ucam.srcf.assassins.exceptions.StateBuilderException;

public class CGIAddDeath implements CGICommand
{
   private void addDeath(CGI cgi, Map plugins, Map params) throws AssassinSQLException, IOException, ParseException, StateBuilderException, CGIInvalidContentFormatException
   {
      // 0 = select event, 1 = select killer & victim, 2 = plugin questions, 3 = commit
      if (null != params.get("reset")) {
         umpirecgi.currentAuth.unsetData("ad_event");
         umpirecgi.currentAuth.unsetData("ad_assassin");
      }
      int pass = 0;
      Event e = null;
      if (null !=  params.get("1") && !"".equals(params.get("1")))
         e = Event.getByID(Integer.parseInt((String) params.get("1")));
      if (null != umpirecgi.currentAuth.getData("ad_event"))
         e = Event.getByID(Integer.parseInt(umpirecgi.currentAuth.getData("ad_event")));
      if (null != params.get("event")) 
         e = (Event) params.get("event");
      
      Assassin victim = null;
      if (null != umpirecgi.currentAuth.getData("ad_victim"))
         victim = Assassin.getByAid(Integer.parseInt(umpirecgi.currentAuth.getData("ad_victim")));
      if (null != params.get("victim"))
         victim = (Assassin) params.get("victim");
      
      Assassin killer = null;
      if (null != umpirecgi.currentAuth.getData("ad_killer"))
         killer = Assassin.getByAid(Integer.parseInt(umpirecgi.currentAuth.getData("ad_killer")));
      if (null != params.get("killer"))
         killer = (Assassin) params.get("killer");

      Death death = null;
      if (null != umpirecgi.currentAuth.getData("ad_death"))
         death = (Death) StateChange.getByScid(Integer.parseInt(umpirecgi.currentAuth.getData("ad_death")));
      if (null != params.get("death"))
         death = (Death) params.get("death");

      cgi.out("<h2>Add a Death to an Event</h2>");
      HTMLForm form = new HTMLForm("umpire.cgi?add-death", "Add Death");

      if (null == e) pass = 0;
      else if (null == killer || null == victim) pass = 1;
      else if (null == death) pass = 2;
      else pass = 3;

      Collection participants = new Vector();
      
      if (pass > 0) if (!umpirecgi.currentAuth.isAtLeast(AuthLevel.UMPIRE)) {
         List pa = umpirecgi.currentAuth.getPlayerViewer().getAssassins();
         if (Debug.debug) Debug.print(umpirecgi.class, "got player assassins:"+pa);
         Collection ea = Tools.viewerifyList(new Vector(e.getParticipants().values()));
         if (Debug.debug) Debug.print(umpirecgi.class, "got participants:"+ea);
         Iterator i = pa.iterator();
         while (i.hasNext()) {
            Object o = i.next();
            if (ea.contains(o)) participants.add(o);
         }
         if (Debug.debug) Debug.print(umpirecgi.class, "got intersection:"+participants);
         if (participants.size() == 0) {
            cgi.out("<div class=\"errormessage\">");
            cgi.out("  <p><b>Insufficient Privileges</b></p>");
            cgi.out("  <p>You are not a participant in this event!</p>");
            cgi.out("</div>");
            return;
         }
         participants = ea;
      } else
         participants = e.getParticipants().values();

      LightweightStateBuilder lsb = null;
      Map state=null;
      Map statev = null;
      if (pass > 1) {
         lsb = new LightweightStateBuilder(e.getEventTime());
         lsb.registerInterest(killer);
         lsb.registerInterest(victim);
         try {
            state = lsb.generateState();
         } catch (StateBuilderException e1) {
            cgi.out("<div class=\"errormessage\">");
            cgi.out("  <p><b>Internal Error</b></p>");
            cgi.out("  <p>State Building failed! Very bad!!</p>");
            cgi.out("</div>");
            cgi.flush();
            e1.printStackTrace();
            throw new RuntimeException();
         }
         statev = Tools.viewerifyStateMap(state);
      }
      
      switch (pass) {
         case 0:
            cgi.out("<p>");
            cgi.out("  Select Event:");
            cgi.out("</p>");
            List l = Event.getAllEventsWithHeadline();
            Event.preCacheSubClasses(Event.READPARTICIPANTS);
            form.addField(new CanonicalEntityField("event","Event:",l,(CanonicalEntity) null));
            form.display(cgi);
            break;
         case 1:
            cgi.out("<p>");
            cgi.out("  Select Killer and Victim:");
            cgi.out("</p>");
            cgi.out("<p>");
            cgi.out("  <b>NOTE</b>: do not use this unless you actually killed someone.");
            cgi.out("  If you were dead, or they were dead and you shot them, it doesn't");
            cgi.out("  count!");
            cgi.out("</p>");
            form.addField(new DisplayField("Event:", e.getPrintable()));
            umpirecgi.currentAuth.storeData("ad_event", ""+e.getEid());
            form.addField(new CanonicalEntityField("killer","Killer (PERSON WHO HAS BEEN KILLING PEOPLE):",
                     participants,(CanonicalEntity)null));
            form.addField(new CanonicalEntityField("victim","Victim (PERSON WHO HAS BEEN KILLED):",
                     participants,(CanonicalEntity)null));
            form.addField(new SubmitButton("reset", "Reset"));
            form.display(cgi);
            break;
         case 2:
            cgi.out("<p>");
            cgi.out("  Additional Questions:");
            cgi.out("</p>");
            AssassinStateViewer ksv = (AssassinStateViewer) statev.get(killer.getViewer());
            AssassinStateViewer vsv = (AssassinStateViewer) statev.get(victim.getViewer());
            if (!ksv.isAlive() || !vsv.isAlive()) {
               cgi.out("<div class=\"warningmessage\">");
               cgi.out("  <p><b>Something odd is going on!</b></p>");
               if (!ksv.isAlive())
                  cgi.out("  <p>"+killer.getPlayer().getName()+" is dead!</p>");
               if (!vsv.isAlive())
                  cgi.out("  <p>"+victim.getPlayer().getName()+" is dead!</p>");
               cgi.out("</div>");
            }
            umpirecgi.currentAuth.storeData("ad_killer", ""+killer.getAid());
            umpirecgi.currentAuth.storeData("ad_victim", ""+victim.getAid());
            umpirecgi.currentAuth.storeData("ad_event", ""+e.getEid());
            form.addField(new HiddenField("killer", ""+killer));
            form.addField(new HiddenField("victim", ""+victim));
            form.addField(new HiddenField("event", ""+e));
            death = new Death(killer, victim);
            e.addStateChange(death);
            e.writeDetails();
            death.writeDetails();

            try {
               state = lsb.generateState();
            } catch (StateBuilderException e1) {
               cgi.out("<div class=\"errormessage\">");
               cgi.out("  <p><b>Internal Error</b></p>");
               cgi.out("  <p>State Building failed! Very bad!!</p>");
               cgi.out("</div>");
               cgi.flush();
               e1.printStackTrace();
               throw new RuntimeException();
            }
            statev = Tools.viewerifyStateMap(state);

            umpirecgi.currentAuth.storeData("ad_death", ""+death.getScid());
            form.addField(new HiddenField("death", ""+death));
            Iterator ps = Plugin.getPluginIterator(plugins);
            while (ps.hasNext()) {
               Plugin p = (Plugin) ps.next();
               Vector qs = p.stateChangeAdded(death, statev, null);
               Iterator qi = qs.iterator();
               for (int i=0; qi.hasNext(); i++) {
                  Question q = (Question) qi.next();
                  form.addField(CGITools.getPluginField(p, q, i));
                  umpirecgi.currentAuth.addSessionQuestion(p, q);
               }
            }
            form.display(cgi);
         case 3:
            try { cgi.header("Refresh", "2; URL=umpire.cgi?edit-event,"+e.getEid()); } catch (CGIHeaderSentException CGIHSe) {} 

            umpirecgi.currentAuth.unsetData("ad_event");
            umpirecgi.currentAuth.unsetData("ad_victim");
            umpirecgi.currentAuth.unsetData("ad_killer");
            umpirecgi.currentAuth.unsetData("ad_death");

            /* Tell plugins about statechanges and answer their questions */
            ps = Plugin.getPluginIterator(plugins);
            while (ps.hasNext()) {
               Plugin p = (Plugin) ps.next();
               Vector qs = CGITools.parseQuestions(p, params);
               p.stateChangeAdded(death, statev, qs);
            }
            Event.commitAll();
            StateChange.commitAll();
            umpirecgi.currentAuth.clearSessionQuestions();
            CGIGeneratePages.generatePages(cgi, plugins, params, false);
            break;
      }
   }
   
   public void command(CGI cgi, Map plugins, Map params, long nowtime) 
      throws AssassinSQLException, IOException, ParseException, StateBuilderException, CGIInvalidContentFormatException { addDeath(cgi, plugins, params); } 
   public String getName() { return "Add Death to Existing Event"; }
   public String getDescription() { return "Adds a death to an existing event"; }
   public AuthLevel accessLevel () { return AuthLevel.PLAYER; }
   public String getSection() { return "Event"; }
   public boolean isVisible() {	return true; }

}

