/* 
 * CUAG AutoUmpire System
 * 
 * Copyright (C) 2003/2004 Matthew Johnson, Adam Biltcliffe,
 * Michael Cripps, Martin O'Leary, Edward Allcutt and James Osborn
 *
 * This program is free software; you can redistribute it and/or 
 * modify it under the terms of the GNU General Public License 
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version. 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details. 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * To Contact the authors, please email assassins@srcf.ucam.org
 *
 */

package org.ucam.srcf.assassins;

import java.io.IOException;
import java.text.ParseException;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.Vector;


import cx.ath.matthew.cgi.CGI;
import cx.ath.matthew.cgi.CGIInvalidContentFormatException;
import cx.ath.matthew.cgi.HiddenField;
import cx.ath.matthew.cgi.NewTable;
import cx.ath.matthew.cgi.SubmitButton;
import cx.ath.matthew.cgi.CGIHeaderSentException;
import cx.ath.matthew.cgi.CheckBox;
import cx.ath.matthew.cgi.DisplayField;
import cx.ath.matthew.cgi.TextField;

import org.ucam.srcf.assassins.exceptions.AssassinSQLException;
import org.ucam.srcf.assassins.exceptions.StateBuilderException;
import org.ucam.srcf.assassins.xml.Config;

public class CGIAddEvent implements CGICommand
{
   Event addEvent(CGI cgi, Map plugins, Map params, boolean withreport) throws AssassinSQLException, IOException, ParseException , StateBuilderException, CGIInvalidContentFormatException
   { 
      if(Debug.debug)
         Debug.print(CGIAddEvent.class, "Pass - "+params.get("pass"));

      // By Mike - fixx0red by matt
      if ("3".equals(params.get("pass"))){ // pass 3
         cgi.out("<h2>Add New Event</h2>");
         cgi.out("<p>");
         cgi.out("Add Death");
         cgi.out("</p>");
         cgi.out("<p>");
         cgi.out("  <b>NOTE</b>: do not use this unless you actually killed someone.");
         cgi.out("  If you were dead, or they were dead and you shot them, it doesn't");
         cgi.out("  count!");
         cgi.out("</p>");

         /* Set the event properties */
         Event e = (Event) params.get("event");
         if(null == e) e = new Event();
         EventViewer ev = (EventViewer) e.getViewer();
         setEventProperties(e, params);

         /* Deal with the pseudonyms and plugin questions about assassins */
         Iterator i = params.keySet().iterator();
         while (i.hasNext()) {
            String key = (String) i.next();
            if (key.startsWith("assassin")) {
               String idx = key.substring(8);
               Assassin a = (Assassin) params.get(key);
               Pseudonym p = (Pseudonym) params.get("pseudonym"+idx);
               int id = Integer.parseInt(idx) + 1;
               if ("on".equals(params.get("nopseud"+idx)))
                  e.setParticipant(id, a, null);
               else
                  e.setParticipant(id, a, p);
            }
         }

         /* build the state */
         Map statev = getViewerStateMap(e,cgi);

         i = params.keySet().iterator();
         while (i.hasNext()) {
            String key = (String) i.next();
            if (key.startsWith("assassin")) {
               String idx = key.substring(8);
               Assassin a = (Assassin) params.get(key);
               Pseudonym p = (Pseudonym) params.get("pseudonym"+idx);
               AssassinViewer av = (AssassinViewer) a.getViewer();
               int id = Integer.parseInt(idx) + 1;
               Iterator ps = Plugin.getPluginIterator(plugins);
               while (ps.hasNext()) {
                  Plugin pl = (Plugin) ps.next();
                  Vector qs = CGITools.parseQuestions(pl, params, (id-1)*100);
                  pl.participantAdded(ev, av, statev, qs);
               }
            }
         }
         Event.commitAll();
         StateChange.commitAll();

         /* Ask for any deaths to be added */
         HTMLForm form=addDeathForm(e, withreport);
         
         /* ??? WTF is it doing here ??? */
         /* Just add it on to the above form for now */
         Iterator plgs = Plugin.getPluginIterator(plugins);
         umpirecgi.currentAuth.clearSessionQuestions();
         while (plgs.hasNext()) {
            Plugin pl = (Plugin) plgs.next();
            Vector qs = pl.eventAdded(ev, statev, null);
            Iterator qi = qs.iterator();
            for (int j=0; qi.hasNext(); j++) {
               Question q = (Question) qi.next();
               form.addField(CGITools.getPluginField(pl, q, j));
               umpirecgi.currentAuth.addSessionQuestion(pl, q);
            }
         }
         Event.commitAll();
         StateChange.commitAll();

         form.display(cgi); 
         return null;
      } else if ("4".equals(params.get("pass")))  { // pass 4
         if ("Add Death".equals(params.get("submit"))) {
            // If we want to add another death
            Event e = (Event) params.get("event");
            EventViewer ev = (EventViewer) e.getViewer();

            /* Build state */
            Map statev = getViewerStateMap(e, cgi);
            
            /* ??? Inform plugins of answer to previous Qs ??? */
            Iterator ps = Plugin.getPluginIterator(plugins);
            while (ps.hasNext()) {
               Plugin p = (Plugin) ps.next();
               Vector qs = CGITools.parseQuestions(p, params);
               p.eventAdded(ev, statev, qs);
            }
            Event.commitAll();
            StateChange.commitAll();
            umpirecgi.currentAuth.clearSessionQuestions();

            /* Add the last death we had */
            Assassin victim = (Assassin) params.get("victim");
            Assassin killer = (Assassin) params.get("killer");
            
            /* Check for Sanity */
            AssassinStateViewer ksv = (AssassinStateViewer) statev.get(killer.getViewer());
            AssassinStateViewer vsv = (AssassinStateViewer) statev.get(victim.getViewer());
            if (!ksv.isAlive() || !vsv.isAlive()) {
               cgi.out("<div class=\"warningmessage\">");
               cgi.out("  <p><b>Something odd is going on</b></p>");
               if (!ksv.isAlive())
                  cgi.out("  <p>"+killer.getPlayer().getName()+" is dead!</p>");
               if (!vsv.isAlive())
                  cgi.out("  <p>"+victim.getPlayer().getName()+" is dead!</p>");
               cgi.out("</div>");
            }

            Death d = new Death(killer,victim);
            e.addStateChange(d);
            Event.commitAll();
            StateChange.commitAll();
            
            /* List the deaths */
            cgi.out("<h2>Add New Event</h2>");
            printDeaths(cgi, e);

            /* Ask additiaonl deaths */
            cgi.out("<p>");
            cgi.out("Add Death, Submit Form to continue without adding another death");
            cgi.out("</p>");
            cgi.out("<p>");
            cgi.out("  <b>NOTE</b>: do not use this unless you actually killed someone.");
            cgi.out("  If you were dead, or they were dead and you shot them, it doesn't");
            cgi.out("  count!");
            cgi.out("</p>");
            addDeathForm(e, withreport).display(cgi); 
         } else {
            // Done adding deaths, let's ask plugins
           HTMLForm form = additionalQuestionsForm(cgi, params, plugins, 
                  (Event) params.get("event"), withreport);
            if(form == null) {
               /* Empty form skip to next phase*/
               params.put("pass", "5");
               return addEvent(cgi, plugins, params, withreport);
            } else {
               cgi.out("<h2>Add New Event</h2>");
               cgi.out("<div class=\"infomessage\">");
               cgi.out("   <p><b>Event Added</b></p>");
               cgi.out("   <p>Additional questions:</p>");
               cgi.out("</div>");
               form.display(cgi);
            }
         }
         return null;
      } else if ("5".equals(params.get("pass"))) {// pass 5
         Event e = (Event) params.get("event");
         if (!withreport) try { cgi.header("Refresh", "2; URL=umpire.cgi?edit-event,"+e.getEid()); } catch (CGIHeaderSentException CGIHSe) {} 
         cgi.out("<h2>Add New Event</h2>");
         cgi.flush();
         e = answerPlugins(e, plugins, params, cgi);
         CGIGeneratePages.generatePages(cgi, plugins, params, false);
         cgi.out("<div class=\"donemessage\">");
         cgi.out("  <p><b>New Event Added Successfully</b></p>");
         cgi.out("  <p>All statechange events added. All questions asked.</p>");
         cgi.out("</div>");
         return e;
      } else if ("2".equals(params.get("pass"))) {// pass 2
         cgi.out("<h2>Add New Event</h2>");
         cgi.out("<p>");
         cgi.out("Select Pseudonyms");
         cgi.out("</p>");
         parseInvolvedGetDetailsForm(params,plugins, withreport, cgi).display(cgi);
         return null;
      } else { // pass 1
         cgi.out("<h2>Add New Event</h2>");
         cgi.out("<p>");
         cgi.out("Select the assassins involved in the event: ");
         cgi.out("</p>");
         selectInvolvedAssassinsForm(withreport).display(cgi);
         return null;
      }
   }


   private static void setEventProperties(Event e, Map params) throws AssassinSQLException, IOException, ParseException , StateBuilderException {
      e.setEventTime(Tools.unFormatTime((String)params.get("event-time")));
      e.setHeadline((String) params.get("headline"));
      e.setActive("on".equals(params.get("active")) ? true : false);
      e.setVisible("on".equals(params.get("visible")) ? true : false);
   }

   private static Map getViewerStateMap(Event e, CGI cgi) throws AssassinSQLException, IOException, ParseException , StateBuilderException, CGIInvalidContentFormatException {
      LightweightStateBuilder lsb = new LightweightStateBuilder(Tools.getNowTime());
      for (Iterator i = e.getParticipants().values().iterator(); i.hasNext();) {
         lsb.registerInterest((Assassin)i.next());
      }

      Map statev = null;
      try {
         statev = Tools.viewerifyStateMap(lsb.generateState());
      } catch (StateBuilderException sbe) {
         printStateBuildError(cgi);
         throw new RuntimeException();
      }

      return statev;
   }

   private static void printDeaths(CGI cgi, Event e) throws AssassinSQLException, IOException, ParseException , StateBuilderException, CGIInvalidContentFormatException {
      cgi.out("<h4>Deaths:</h4>");
      cgi.out("<ul>");
      List l = e.getStateChanges();
      Iterator i = l.iterator();
      while (i.hasNext()) {
         StateChange sc = (StateChange) i.next();
         if (sc instanceof Death)
            cgi.out("   <li>"+((Assassin) sc.getEntity()).getPlayer().getName()+
                  " killed "+sc.getAssassin().getPlayer().getName()+"</li>");
      }
      cgi.out("</ul>");
   }

   private static HTMLForm addDeathForm(Event e, boolean withreport) throws AssassinSQLException, IOException, ParseException , StateBuilderException {
      HTMLForm form;
      if (withreport) 
         form = new HTMLForm("umpire.cgi?add-event-with-report", "No More Deaths");
      else
         form = new HTMLForm("umpire.cgi?add-event", "No More Deaths");
      form.addField(new HiddenField("event",""+e));
      form.addField(new HiddenField("pass","4"));

      form.addField(new CanonicalEntityField("killer","Select killer (PERSON WHO HAS BEEN KILLING PEOPLE):",e.getParticipants().values(),(CanonicalEntity) null));
      form.addField(new CanonicalEntityField("victim","Select victim (DEAD PERSON):",e.getParticipants().values(),(CanonicalEntity) null));
      form.addField(new SubmitButton("submit","Add Death"));
      return form;
   }

   private static HTMLForm additionalQuestionsForm(CGI cgi, Map params, Map plugins, Event e, boolean withreport) throws AssassinSQLException, IOException, ParseException , StateBuilderException, CGIInvalidContentFormatException {
      HTMLForm form;
      if (withreport) 
         form = new HTMLForm("umpire.cgi?add-event-with-report", "Answer");
      else
         form = new HTMLForm("umpire.cgi?add-event", "Answer");
      form.addField(new HiddenField("pass", "5"));

      Map statev = getViewerStateMap(e,cgi);
      EventViewer ev = (EventViewer) e.getViewer();
      Iterator ps = Plugin.getPluginIterator(plugins);
      while (ps.hasNext()) {
         Plugin p = (Plugin) ps.next();
         Vector qs = CGITools.parseQuestions(p, params);
         p.eventAdded(ev, statev, qs);
      }
      Event.commitAll();
      StateChange.commitAll();
      umpirecgi.currentAuth.clearSessionQuestions();

      boolean qsAsked=false;    /*monitor if its in fact an empty form */
      Iterator scs = e.getStateChanges().iterator();
      for (int idx = 0; scs.hasNext(); idx++) {
         StateChange sc = (StateChange) scs.next();
         ps = Plugin.getPluginIterator(plugins);
         while (ps.hasNext()) {
            Plugin p = (Plugin) ps.next();
            Vector qs = p.stateChangeAdded(sc, statev, null);
            Iterator qi = qs.iterator();
            for (int i=idx*100; qi.hasNext(); i++) {
               qsAsked=true;
               Question q = (Question) qi.next();
               form.addField(CGITools.getPluginField(p, q, i));
               umpirecgi.currentAuth.addSessionQuestion(p, q);
            }
         }
      }
      form.addField(new HiddenField("event", ""+e));
      if(!qsAsked)
         return null;
      return form;
   }

   private static Event answerPlugins(Event e, Map plugins, Map params, CGI cgi) throws AssassinSQLException, IOException, ParseException , StateBuilderException, CGIInvalidContentFormatException {
      /* Build state for participants */
      Map statev = getViewerStateMap(e,cgi);

      /* Tell plugins about statechanges and answer their questions */
      Iterator scs = e.getStateChanges().iterator();
      for (int idx = 0; scs.hasNext(); idx++) {
         StateChange sc = (StateChange) scs.next();
         Iterator ps = Plugin.getPluginIterator(plugins);
         while (ps.hasNext()) {
            Plugin p = (Plugin) ps.next();
            Vector qs = CGITools.parseQuestions(p, params, idx*100);
            p.stateChangeAdded(sc, statev, qs);
         }
      }
      Event.commitAll();
      StateChange.commitAll();
      umpirecgi.currentAuth.clearSessionQuestions();

      return e;
   }

   private static HTMLForm parseInvolvedGetDetailsForm(Map params, Map plugins, boolean withreport, CGI cgi) throws AssassinSQLException, IOException, ParseException , StateBuilderException, CGIInvalidContentFormatException {
      /* Get the list of assassins involved in this event */
      Object o = params.get("assassins");
      List as = new LinkedList();
      if (o instanceof List) 
         as = (List) o;
      else if (o instanceof Assassin)
         as.add(o);
      
      /* Build state for said assassins */
      LightweightStateBuilder lsb = new LightweightStateBuilder(Tools.getNowTime());
      for (Iterator i=as.iterator(); i.hasNext();) {
         lsb.registerInterest((Assassin)i.next());
      }
      Map myState = null;
      try {
         myState=lsb.generateState();
      } catch (StateBuilderException sbe) {
         printStateBuildError(cgi); 
         throw new RuntimeException();
      }
      Map statev = Tools.viewerifyStateMap(myState);

      /* Create the Event to add stuff to */
      Event e = new Event();
      e.writeDetails();
      EventViewer ev = (EventViewer) e.getViewer();
      
      /* For each assassin ask pseudonym to use and get plugin questions */
      Iterator i = as.iterator();
      int idx=0;
      HTMLForm form;
      if (withreport) 
         form = new HTMLForm("umpire.cgi?add-event-with-report", "Add");
      else
         form = new HTMLForm("umpire.cgi?add-event", "Add");
      form.addField(new HiddenField("pass","3"));
	  form.addField(new NewTable("new player "+idx,"PlayerDetails"));
      Vector qs;
      while (i.hasNext()) {
         Assassin a;
         a = (Assassin) i.next();
         AssassinViewer av = (AssassinViewer) a.getViewer();
		 form.addField(new HiddenField("assassin"+idx,""+a));
         form.addField(new CanonicalEntityField("pseudonym"+idx,(idx+1)+"."+a.getPlayer().getName(),((AssassinState) myState.get(a)).getPseudonyms(),(CanonicalEntity) null));
         form.addField(new CheckBox("nopseud"+idx, "Use Real Name", false));
         Iterator j = Plugin.getPluginIterator(plugins);
         while (j.hasNext()) {
            Plugin p = (Plugin) j.next();
            qs = p.participantAdded(ev, av, statev, null);
            Iterator qi = qs.iterator();
            for (int k=idx*100; qi.hasNext(); k++) {
               Question q = (Question) qi.next();
               form.addField(CGITools.getPluginField(p, q, k));
               umpirecgi.currentAuth.addSessionQuestion(p, q);
            }
         }
         ++idx;
		 if (i.hasNext()) {
			 form.addField(new NewTable("new player "+idx,"PlayerDetails"));
		 } else {
			 form.addField(new NewTable("event details", "EventDetails"));
		 }
      }
      Event.commitAll();
      StateChange.commitAll();

      /* Get details about the event */
      form.addField(new TextField("event-time","Event Time:", Tools.formatTime(Tools.getNowTime())));
      String defheadline = Config.getConfig("vanilla", "default-headline");
      form.addField(new TextField("headline","Headline:", null==defheadline?"":defheadline, 40));
      form.addField(new DisplayField("",
               "Headlines may contain the special sequences "+
               "[Pn], [Nn] and [Ln], "+
               "where n is the participant number.<br/> "+
               "[Pn] will be replaced by their pseudonym, "+
               "[Nn] by their name "+
               "and [Ln] by the list of all their pseudonyms."));
      if (umpirecgi.currentAuth.isAtLeast(AuthLevel.UMPIRE)) {
         form.addField(new CheckBox("active","Make Active?", true));
         form.addField(new CheckBox("visible","Make Visible?", true));
      } else {
         form.addField(new HiddenField("active","on"));
         form.addField(new HiddenField("visible","on"));
      }
      form.addField(new HiddenField("event",""+e));
      Event.commitAll();

      return form;
   }
   
   private static void printStateBuildError(CGI cgi) throws IOException, CGIInvalidContentFormatException {
      cgi.out("<div class=\"errormessage\">");
      cgi.out("  <p><b>Internal Error</b></p>");
      cgi.out("  <p>State Building failed! Very bad!!</p>");
      cgi.out("</div>");
      cgi.flush();
   }

   private static HTMLForm selectInvolvedAssassinsForm(boolean withreport) throws AssassinSQLException, IOException, ParseException , StateBuilderException {
      List as = Assassin.getAllAssassinsOrderByRealName();
      if (Debug.debug) Debug.print(CGIAddEvent.class,  "assassins got: "+as+" ("+as.size()+")");
      HTMLForm form;
      if (withreport) 
         form = new HTMLForm("umpire.cgi?add-event-with-report", "Add");
      else
         form = new HTMLForm("umpire.cgi?add-event", "Add");
      form.addField(new MultipleCanonicalEntityField("assassins","",as,(CanonicalEntity) null));
      form.addField(new HiddenField("pass","2"));
      return form;
   }


   public void command(CGI cgi, Map plugins, Map params, long nowtime) 
      throws AssassinSQLException, IOException, ParseException, StateBuilderException, CGIInvalidContentFormatException {  addEvent(cgi, plugins, params, false); } 
   public String getName() { return "Add Event"; }
   public String getDescription() { return "Adds an event"; }
   public AuthLevel accessLevel () { return AuthLevel.PLAYER; }
   public String getSection() { return "Event"; }
   public boolean isVisible() {	return true; }
}

