/* 
 * CUAG AutoUmpire System
 * 
 * Copyright (C) 2003/2004 Matthew Johnson, Adam Biltcliffe,
 * Michael Cripps, Martin O'Leary, Edward Allcutt and James Osborn
 *
 * This program is free software; you can redistribute it and/or 
 * modify it under the terms of the GNU General Public License 
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version. 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details. 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * To Contact the authors, please email assassins@srcf.ucam.org
 *
 */

package org.ucam.srcf.assassins;

import java.io.IOException;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.Map;
import java.util.Iterator;
import java.io.PipedOutputStream;
import java.io.PipedInputStream;
import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.io.IOException;

import cx.ath.matthew.cgi.CGI;
import cx.ath.matthew.cgi.CGIInvalidContentFormatException;
import cx.ath.matthew.cgi.TextField;

import org.ucam.srcf.assassins.exceptions.AssassinSQLException;
import org.ucam.srcf.assassins.xml.Config;

public class CGIDbBackup implements CGICommand
{
   void dbBackup(CGI cgi, Map plugins, Map params) throws CGIInvalidContentFormatException, IOException
   {
      cgi.out("<h2>Database Backup</h2>");
      if (null == params.get("submit")) {
         cgi.out("<p>");
         cgi.out("  This function allows you to do a dump of the database.");
         cgi.out("</p>");
         HTMLForm form = new HTMLForm("umpire.cgi?backup", "Backup");
         String defpath = (String) Config.getConfig("vanilla", "backup-path");
         form.addField(new TextField("path", "Directory to put dump file", null==defpath ? "" : defpath));
         form.display(cgi);
      } else {
         String dbtype = Config.getConfig("vanilla", "dbtype");
         String path = (String) params.get("path");
         if (!"mysql".equals(dbtype)) {
            cgi.out("<p>");
            cgi.out("  Can only dump MySQL databases");
            cgi.out("</p>");
            return;
         }
         if (null == path || "".equals("path")) {
            cgi.out("<p>");
            cgi.out("  Please enter a path!");
            cgi.out("</p>");
            return;
         }
         
         cgi.out("<p>");
         cgi.out("   Running dump...");
         cgi.flush();
         String filename = null;
         try {
            SimpleDateFormat sdf = new SimpleDateFormat("yyMMdd-HHmms");
            Date d = new Date();
            String prefix = DatabaseBackend.getPrefix();
            filename = "autoumpiredb."+prefix+"."+sdf.format(d)+".sql";

            PipedInputStream inp = new PipedInputStream();
            PipedOutputStream outp = new PipedOutputStream(inp);
            BufferedReader inr = new BufferedReader(new InputStreamReader(inp));
            PipedInputStream einp = new PipedInputStream();
            PipedOutputStream eoutp = new PipedOutputStream(einp);
            BufferedReader err = new BufferedReader(new InputStreamReader(einp));
            
            String dbhost = Config.getConfig("vanilla", "dbhost");
            String dbport = Config.getConfig("vanilla", "dbport");
            String dbname = Config.getConfig("vanilla", "dbname");
            String dbuser = Config.getConfig("vanilla", "dbuser");
            String dbpass = Config.getConfig("vanilla", "dbpass");
            
            Tools.exec(new String[] {
               "/bin/sh",
               "-c",
               "/usr/bin/mysqldump "+
               " -h "+dbhost+
               " -u "+dbuser+
               " -p"+dbpass+ // this lack of a space is *really important* it *doesn't work* without it
               " -P "+dbport+
               " "+dbname+" "+
               prefix+"_assassins "+
               prefix+"_players "+
               prefix+"_statechanges "+
               prefix+"_events "+
               prefix+"_reports "+
               prefix+"_eventparticipants "+
               prefix+"_pseudonyms "+
               prefix+"_active_sessions "+
               prefix+"_session_data "+
               prefix+"_session_other_data "+
               " > "+
               path+"/"+filename
            }, eoutp);

            while (err.ready()) cgi.out(err.readLine());
         } catch (IOException IOe) {
            cgi.out("   Dump of database failed!");
         }
         cgi.out("</p>");
         cgi.out("<p>");
         cgi.out("  The database has been dumped to <tt>"+path+"/"+filename+"</tt>.");
         cgi.out("</p>");
      }
   }

   public void command(CGI cgi, Map plugins, Map params, long nowtime) 
      throws AssassinSQLException, IOException, ParseException, CGIInvalidContentFormatException { dbBackup(cgi, plugins, params); } 
   public String getName() { return "Database Backup"; }
   public String getDescription() { return "Dumps the database to a file"; }
   public AuthLevel accessLevel () { return AuthLevel.UMPIRE; }
   public String getSection() { return "Misc"; }
   public boolean isVisible() {	return true; }
}

