/* 
 * CUAG AutoUmpire System
 * 
 * Copyright (C) 2003/2004 Matthew Johnson, Adam Biltcliffe,
 * Michael Cripps, Martin O'Leary, Edward Allcutt and James Osborn
 *
 * This program is free software; you can redistribute it and/or 
 * modify it under the terms of the GNU General Public License 
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version. 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details. 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * To Contact the authors, please email assassins@srcf.ucam.org
 *
 */

package org.ucam.srcf.assassins;

import java.io.IOException;
import java.text.ParseException;
import java.util.Collection;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.TreeSet;

import cx.ath.matthew.cgi.CGI;
import cx.ath.matthew.cgi.CGIHeaderSentException;
import cx.ath.matthew.cgi.CGIInvalidContentFormatException;
import cx.ath.matthew.cgi.CheckBox;
import cx.ath.matthew.cgi.DisplayField;
import cx.ath.matthew.cgi.HiddenField;
import cx.ath.matthew.cgi.TextField;

import org.ucam.srcf.assassins.exceptions.AssassinSQLException;
import org.ucam.srcf.assassins.exceptions.StateBuilderException;
import org.ucam.srcf.assassins.xml.Config;

public class CGIEditEvent implements CGICommand
{
   private void editEvent(CGI cgi, Map plugins, Map params) throws AssassinSQLException, IOException, ParseException, StateBuilderException, CGIInvalidContentFormatException
   {
      if (null != params.get("update")) {
         Event e = (Event) params.get("event");
         e.setHeadline((String) params.get("headline"));
         if (umpirecgi.currentAuth.isAtLeast(AuthLevel.UMPIRE)) {
            e.setEventTime(Tools.unFormatTime((String) params.get("time")));
            e.setActive("on".equals(params.get("active")) ? true : false);
            e.setVisible("on".equals(params.get("visible")) ? true : false);
         }
         Event.commitAll();
         cgi.out("<h2>Edit Event</h2>");
         cgi.out("<div class=\"donemessage\">");
         cgi.out("   <p><b>Event Updated</b></p>");
         cgi.out("   <p>&nbsp;</p>");
         cgi.out("</div>");
         try { cgi.header("Refresh", "2; URL=umpire.cgi?edit-event,"+e.getEid()); } catch (CGIHeaderSentException CGIHSe) {} 
         CGIGeneratePages.generatePages(cgi, plugins, params, false);
      } 
      else {
         Event e = (Event) params.get("event");
         if (null == e) {
            String s = (String) params.get("1");
            if (null != s && !"".equals(s)) {
               int eid = Integer.parseInt(s);
               if ("public".equals(params.get("2"))) eid -= Config.getConfigInt("vanilla","obscurity-offset"); 
               e = Event.getByID(eid);
            }
         }
         cgi.out("<h2>Edit Event</h2>");
         if (null == e) {
            cgi.out("<p>");
            cgi.out("   Select the Event to edit.");
            cgi.out("</p>");
            HTMLForm form = new HTMLForm("umpire.cgi?edit-event", "Edit");
            List l = Event.getAllEventsWithHeadline();
            Event.preCacheSubClasses(Event.READPARTICIPANTS);
            form.addField(new CanonicalEntityField("event","Event:",l,e));
            form.display(cgi);
         } 
         else {
            cgi.out("<p>");
            cgi.out("   Edit the event details, and click below to alter participants and reports.");
            cgi.out("</p>");
            HTMLForm form = new HTMLForm("umpire.cgi?edit-event", "Edit");
            form.addField(new TextField("headline", "Headline: ", e.getHeadline(), 60));
            form.addField(new DisplayField("",
                     "Headlines may contain the special sequences "+
                     "[Pn], [Nn] and [Ln], "+
                     "where n is the participant number.<br/> "+
                     "[Pn] will be replaced by their pseudonym, "+
                     "[Nn] by their name "+
                     "and [Ln] by the list of all their pseudonyms."));
            if (umpirecgi.currentAuth.isAtLeast(AuthLevel.UMPIRE)) {
               form.addField(new TextField("time", "Event Time: ", Tools.formatTime(e.getEventTime())));
               form.addField(new CheckBox("active", "Active: ", e.isActive()));
               form.addField(new CheckBox("visible", "Visible: ", e.isVisible()));
            } else {
               form.addField(new DisplayField("Event Time: ", Tools.formatTime(e.getEventTime())));
               form.addField(new DisplayField("Active: ", e.isActive()?"yes":"no"));
               form.addField(new DisplayField("Visible: ", e.isVisible()?"yes":"no"));
            }
            form.addField(new HiddenField("event", ""+e));
            form.addField(new HiddenField("update", "true"));
            form.display(cgi);

            LightweightStateBuilder lsb = new LightweightStateBuilder(e.getEventTime());
            Iterator i = e.getParticipants().values().iterator();
            while (i.hasNext()) 
               lsb.registerInterest((Assassin) i.next());
            Map state=null;
            try {
               state = lsb.generateState();
            } catch (StateBuilderException e1) {
               cgi.out("<div class=\"errormessage\">");
               cgi.out("  <p><b>Internal Error</b></p>");
               cgi.out("  <p>State Building failed! Very bad!!</p>");
               cgi.out("</div>");
               cgi.flush();
               e1.printStackTrace();
               throw new RuntimeException();
            }

            printParticipants(e, cgi, state, plugins);
            
            printDeaths(e, cgi);      
            
            printReports(e, cgi);
         }
      }
   }
   private void printReports(Event e, CGI cgi) throws AssassinSQLException, CGIInvalidContentFormatException
   {
      cgi.out("<h4>Reports</h4>");
      cgi.out("<ul>");
      Iterator i = e.getReports().iterator();
      while (i.hasNext()) {
         Report r = (Report) i.next();
         Assassin a = r.getAssassin();
         cgi.out(" <li>"+r.getPrintable());
         if (umpirecgi.currentAuth.isAtLeast(AuthLevel.UMPIRE) || 
               (null != a && 
                a.getPlayer().equals(umpirecgi.currentAuth.getPlayerViewer().getEntity()))) {
            cgi.out("[");
            CGITools.printlink(cgi, "rep"+r.getRid(), 
                  "umpire.cgi?edit-report,"+r.getRid(), "Edit");    
            cgi.out("] [");
            CGITools.printlink(cgi, "rep"+r.getRid(), 
                  "umpire.cgi?remove-report,"+r.getRid(), "Delete");    
            cgi.out("]");
         }
         cgi.out("</li>");
      }
      cgi.out(" <li>");
      CGITools.printlink(cgi, "addr", 
            "umpire.cgi?add-report,"+e.getEid(), "Add New Report");    
      cgi.out(" </li>");
      cgi.out("</ul>");

   }
   private void printParticipants(Event e, CGI cgi, Map state, Map plugins) throws AssassinSQLException, CGIInvalidContentFormatException
   {
      cgi.out("<h4>Participants</h4>");
      cgi.out("<ul>");
      Iterator i = new TreeSet(e.getParticipants().keySet()).iterator();
      Collection participants = e.getParticipants().values();
      while (i.hasNext()) {
         int n = ((Integer) i.next()).intValue();
         Assassin a = e.getParticipant(n);
         AssassinState ast = (AssassinState) state.get(a);
         Pseudonym p = e.getParticipantPseudonym(n);
         cgi.out(" <li>"+n+". ");
            CGITools.printlink(cgi, "edita"+n, 
                  "umpire.cgi?edit-assassin,"+a.getAid(),
                  a.getPrintable(), 
                  "colour"+Tools.getColourForAssassin(plugins, ast, 
                     a.getPlayer().getName()));
         cgi.out(" as "+(null==p?a.getPlayer().getName():p.getPrintable()));
         if (umpirecgi.currentAuth.isAtLeast(AuthLevel.UMPIRE) || 
               a.getPlayer().equals(umpirecgi.currentAuth.getPlayerViewer().getEntity())) {
            cgi.out("[");
            CGITools.printlink(cgi, "editps"+n, "umpire.cgi?change-participant-pseudonym,"+e.getEid()+","+n,"Change");
            cgi.out("]");
         }
         cgi.out("</li>");
      }
      cgi.out(" <li>");
      CGITools.printlink(cgi, "addp", "umpire.cgi?add-participant,"+e.getEid(), "Add Participant");    
      cgi.out(" </li>");
      cgi.out("</ul>");
   }
   private void printDeaths(Event e, CGI cgi) throws AssassinSQLException, CGIInvalidContentFormatException
   {
      cgi.out("<h4>Deaths</h4>");
      cgi.out("<ul>");
      Iterator i = e.getStateChanges().iterator();
      while (i.hasNext()) {
         Object o = i.next();
         if (o instanceof Death) {
            Death d = (Death) o;
            cgi.out(" <li>"+d.getKiller().getPlayerViewer().getName()+
                  " killed "+d.getVictim().getPlayerViewer().getName());
            if (umpirecgi.currentAuth.isAtLeast(AuthLevel.UMPIRE)) {
               cgi.out(" [");
               CGITools.printlink(cgi, "remd", "umpire.cgi?remove-statechange,"+e.getEid()+","+d.getScid(), "Remove");    
               cgi.out(" ]");
            }
            cgi.out("</li>");
         }
      }
      cgi.out(" <li>");
      CGITools.printlink(cgi, "addd", "umpire.cgi?add-death,"+e.getEid(), "Add Death");    
      cgi.out(" </li>");
      cgi.out("</ul>");
   }
   public void command(CGI cgi, Map plugins, Map params, long nowtime) 
      throws AssassinSQLException, IOException, ParseException, StateBuilderException, CGIInvalidContentFormatException {  editEvent(cgi, plugins, params); } 
   public String getName() { return "Edit Event"; }
   public String getDescription() { return "Edits the event details and allows you to add/modify reports and add participants"; }
   public AuthLevel accessLevel () { return AuthLevel.PLAYER; }
   public String getSection() { return "Event"; }
   public boolean isVisible() {	return true; }
}

