/* 
 * CUAG AutoUmpire System
 * 
 * Copyright (C) 2003/2004 Matthew Johnson, Adam Biltcliffe,
 * Michael Cripps, Martin O'Leary, Edward Allcutt and James Osborn
 *
 * This program is free software; you can redistribute it and/or 
 * modify it under the terms of the GNU General Public License 
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version. 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details. 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * To Contact the authors, please email assassins@srcf.ucam.org
 *
 */

package org.ucam.srcf.assassins;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.PrintWriter;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.Map;

import cx.ath.matthew.cgi.CGI;
import cx.ath.matthew.cgi.CGIHeaderSentException;
import cx.ath.matthew.cgi.CGIInvalidContentFormatException;
import cx.ath.matthew.cgi.CheckBox;
import cx.ath.matthew.cgi.HiddenField;
import cx.ath.matthew.cgi.TextArea;
import cx.ath.matthew.cgi.TextField;

import org.ucam.srcf.assassins.exceptions.AssassinSQLException;
import org.ucam.srcf.assassins.exceptions.StateBuilderException;

public class CGIGenerateMail implements CGICommand
{
   private void generateMail(CGI cgi, Map plugins, Map params)  throws AssassinSQLException, IOException, ParseException , StateBuilderException, CGIInvalidContentFormatException
   {
      // build the email file
      if ("2".equals(params.get("pass"))) {
         boolean force = false;
         String manualText = "";
         String outFile = null;
         force = params.containsKey("force");
         manualText = (String) params.get("text");
         outFile = (String) params.get("output");

         if (null == outFile) outFile = "email";

         cgi.out("<h2>Send Emails To Players</h2>");
         cgi.out("<p>");
         cgi.out("   Generating State, please wait....");
         cgi.out("<p>");
         cgi.flush();

         StateBuilder sb = new StateBuilder(plugins);
         Map state;
         try {
            state = sb.buildStateAtTime(Tools.getNowTime());
         } catch (StateBuilderException sbe) {
            cgi.out("<div class=\"errormessage\">");
            cgi.out("  <p><b>Internal Error</b></p>");
            cgi.out("  <p>State Building failed! Very bad!!</p>");
            cgi.out("</div>");
            cgi.flush();
            throw new RuntimeException();
         }
         String emails = Tools.getEmailText(state, plugins, force, manualText);
         cgi.out("<p>");
         cgi.out("   The following emails will be sent to players, and has been stored in the file '"+outFile+"'.");
         cgi.out("</p>");
         HTMLForm form = new HTMLForm("umpire.cgi?generate-mail", "Send Emails");
         form.addField(new HiddenField("pass", "3"));
         form.addField(new HiddenField("output", outFile));
         form.display(cgi);
         File o = new File(outFile);
         PrintWriter out = new PrintWriter(new FileOutputStream(o));
         out.println(emails);
         out.flush();
         out.close();
         cgi.out("<hr/>");
         cgi.out("<pre>");
         cgi.out(emails);
         cgi.out("</pre>");
         cgi.out("<hr/>");
      }
      // send emails
      else if ("3".equals(params.get("pass"))) {
         String outFile = null;
         outFile = (String) params.get("output");
         if (null == outFile) outFile = "email";

         Tools.exec(new String[] {
            "/bin/sh",
            "-c",
            "/usr/sbin/sendmail -bS < "+outFile,
         });

         cgi.out("<h2>Send Emails To Players</h2>");
         cgi.out("<div class=\"donemessage\">");
         cgi.out("  <p><b>Email Sent</b></p>");
         cgi.out("  <p>&nbsp;</p>");
         cgi.out("</div>");
         try { cgi.header("Refresh", "2; URL=umpire.cgi"); } catch (CGIHeaderSentException CGIHSe) {} 
      }
      // ask about it
      else {
         cgi.out("<h2>Send Emails To Players</h2>");
         cgi.out("<p>");
         cgi.out("   By default this will email all players with outstanding information they need to be sent, such as new targets, "+
               "password changes or competency warnings.");
         cgi.out("   You can also force an email to go to all players. All emails will contain the player details as a check.");
         cgi.out("</p>");

         HTMLForm form = new HTMLForm("umpire.cgi?generate-mail", "Generate Emails");
         form.addField(new TextField("output", "File to save emails to:", "email."+new SimpleDateFormat("yyyyMMdd").format(new Date())));
         form.addField(new CheckBox("force", "Send email to all players:", false));
         form.addField(new TextArea("text", "Extra text to send to players:", ""));
         form.addField(new HiddenField("pass", "2"));
         form.display(cgi);
      }
   }
   public void command(CGI cgi, Map plugins, Map params, long nowtime) 
      throws AssassinSQLException, IOException, ParseException, StateBuilderException, CGIInvalidContentFormatException {  generateMail(cgi, plugins, params); }
   public String getName() { return "Generate Mail"; }
   public String getDescription() { return "Generates emails to be sent to the players if they need it"; }
   public AuthLevel accessLevel () { return AuthLevel.UMPIRE; }
   public String getSection() { return "Updates"; }
   public boolean isVisible() {	return true; }
}

