/* 
 * CUAG AutoUmpire System
 * 
 * Copyright (C) 2003/2004 Matthew Johnson, Adam Biltcliffe,
 * Michael Cripps, Martin O'Leary, Edward Allcutt and James Osborn
 *
 * This program is free software; you can redistribute it and/or 
 * modify it under the terms of the GNU General Public License 
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version. 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details. 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * To Contact the authors, please email assassins@srcf.ucam.org
 *
 */

package org.ucam.srcf.assassins;

import java.io.IOException;
import java.text.ParseException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Vector;

import cx.ath.matthew.cgi.CGI;
import cx.ath.matthew.cgi.CGIHeaderSentException;
import cx.ath.matthew.cgi.CGIInvalidContentFormatException;
import cx.ath.matthew.cgi.HiddenField;

import org.ucam.srcf.assassins.exceptions.AssassinSQLException;
import org.ucam.srcf.assassins.exceptions.StateBuilderException;

public class CGIUpdateState implements CGICommand
{
   private void updateState(CGI cgi, Map plugins, Map params) throws AssassinSQLException, IOException, ParseException, StateBuilderException, CGIInvalidContentFormatException
   {
      cgi.out("<h2>Update State</h2>");
      ((umpirecgi) cgi).setTab("Functions");
      cgi.out("<p>");
      cgi.out("   Calculating state, please wait....");
      cgi.out("</p>");
      cgi.flush();

      cgi.out("<p>");
      cgi.out("Vanilla...");
      cgi.flush();
      if (Debug.debug) Debug.print(umpirecgi.class, "Building State...");
      // build our state
      StateBuilder sb = new StateBuilder(plugins);
      Map state;
      try {
         state = sb.buildStateAtTime(Tools.getNowTime());
      } catch (StateBuilderException sbe) {
         cgi.out("<div class=\"errormessage\">");
         cgi.out("  <p><b>Internal Error</b></p>");
         cgi.out("  <p>State Building failed! Very bad!!</p>");
         cgi.out("</div>");
         cgi.flush();
         throw new RuntimeException();
      }
      Map statev = Tools.viewerifyStateMap(state);

      if (Debug.debug) Debug.print(umpirecgi.class, "Get Event Parameter");
      // get the event we are using
      Event e = (Event) params.get("event");
      if (null == e) {
         e = new Event();
         e.setEventTime(Tools.getNowTime());
         e.setModifiedTime(Tools.getNowTime());
         e.setActive(true);
         e.setVisible(false);
         e.writeDetails();
      }
      EventViewer ev = (EventViewer) e.getViewer();

      boolean haveqs = false;
      HTMLForm form = new HTMLForm("umpire.cgi?update-state", "Submit");
      form.addField(new HiddenField("event",""+e));
      
      cgi.out("<i>done</i>");
      cgi.out("</p>");
      cgi.flush();

      // make sure we damn well read everything out of the db in advance
      StateChange.preCache();
      Event.getAllEvents();
      Event.preCacheSubClasses(Event.READALL);

      // get the questions we asked last time, and ask for any more
      Iterator i = Plugin.getPluginIterator(plugins);
      while (i.hasNext()) {
         Plugin p = (Plugin) i.next();
         cgi.out("<p>");
         cgi.out(p.getPluginName()+"...");
         cgi.flush();
         if (Debug.debug) Debug.print(umpirecgi.class, "Got plugin "+p.getPluginName());
         if (Debug.debug) Debug.print(umpirecgi.class, "Parsing Questions");
         Vector qs = CGITools.parseQuestions(p, params);
         if (Debug.debug) Debug.print(umpirecgi.class, "Updating State");
         qs = p.updateState(statev, (Vector) qs, ev); 
         if (Debug.debug) Debug.print(umpirecgi.class, "Clearing Session");
         umpirecgi.currentAuth.clearSessionQuestions(p);
         if (null != qs && qs.size() > 0) {
            haveqs = true;
            Iterator j = qs.iterator();
            for (int k = 0; j.hasNext(); k++) {
               Question q = (Question) j.next();
               if (Debug.debug) Debug.print(umpirecgi.class, "Adding question "+q);
               form.addField(CGITools.getPluginField(p, q, k));
               umpirecgi.currentAuth.addSessionQuestion(p, q);
            }
         }
         if (Debug.debug) Debug.print(umpirecgi.class, "Committing results");
         DatabaseBackend.commitAll();
         cgi.out("<i>done</i>");
         cgi.out("</p>");
         cgi.flush();
      }

      if (haveqs) {
         cgi.out("<p>");
         cgi.out("   Plugins have questions to ask:");
         cgi.out("</p>");
         form.display(cgi);   
      }
      else {
         cgi.out("<div class=\"donemessage\">");
         cgi.out("   <p><b>All Questions Asked</b></p>");
         cgi.out("   <p>State Updated.</p>");
         cgi.out("</div>");
         try { cgi.header("Refresh", "2; URL=umpire.cgi"); } catch (CGIHeaderSentException CGIHSe) {} 
      }
   }

   public void command(CGI cgi, Map plugins, Map params, long nowtime) 
      throws AssassinSQLException, IOException, ParseException, StateBuilderException, CGIInvalidContentFormatException {  updateState(cgi, plugins, params); }
   public String getName() { return "Update State"; }
   public String getDescription() { return "Allows all the plugins to check their state and ask any questions they need to"; }
   public AuthLevel accessLevel () { return AuthLevel.UMPIRE; }
   public String getSection() { return "Updates"; }
   public boolean isVisible() {	return true; }
}

