/* 
 * CUAG AutoUmpire System
 * 
 * Copyright (C) 2003/2004 Matthew Johnson, Adam Biltcliffe,
 * Michael Cripps, Martin O'Leary, Edward Allcutt and James Osborn
 *
 * This program is free software; you can redistribute it and/or 
 * modify it under the terms of the GNU General Public License 
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version. 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details. 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * To Contact the authors, please email assassins@srcf.ucam.org
 *
 */

package org.ucam.srcf.assassins;

import java.util.Collection;
import java.util.Iterator;
import java.util.List;
import java.util.Vector;

import org.ucam.srcf.assassins.exceptions.AssassinSQLException;
import org.ucam.srcf.assassins.exceptions.NotSerialisedEntityException;

/**
 * A class for showing a dropdown of entities, and recovering the entity afterwards
 */
public class CanonicalEntityField extends cx.ath.matthew.cgi.Field
{
   CanonicalEntity[] entities;
   CanonicalEntity defentity;
   boolean indexed = false;
     
   public CanonicalEntityField(String name, String label, Collection entities, CanonicalEntity defentity, boolean indexed)
   {
      this.name = name;
      this.label = label;
      if (entities.size() == 0)
         this.entities = new CanonicalEntity[]{};
      else {
         Iterator i = entities.iterator();
         Object o = i.next();
         if (o instanceof CanonicalEntity)
            this.entities = (CanonicalEntity[]) (entities.toArray(new CanonicalEntity[] {}));
         else if (o instanceof CanonicalViewer) {
            List a = new Vector();
            a.add(o);
            while (i.hasNext())
               a.add(((Viewer) i.next()).getEntity());
            this.entities = (CanonicalEntity[]) (a.toArray(new CanonicalEntity[] {}));
         } else
            this.entities = new CanonicalEntity[]{};
      }
      this.defentity = defentity;
      this.indexed = indexed;
   }

   public CanonicalEntityField(String name, String label, Collection entities, CanonicalViewer defentity, boolean indexed)
   {
      this.name = name;
      this.label = label;
      if (entities.size() == 0)
         this.entities = new CanonicalEntity[]{};
      else {
         Iterator i = entities.iterator();
         Object o = i.next();
         if (o instanceof CanonicalEntity)
            this.entities = (CanonicalEntity[]) (entities.toArray(new CanonicalEntity[] {}));
         else if (o instanceof CanonicalViewer) {
            List a = new Vector();
            a.add(((Viewer) o).getEntity());
            while (i.hasNext())
               a.add(((Viewer) i.next()).getEntity());
            this.entities = (CanonicalEntity[]) (a.toArray(new CanonicalEntity[] {}));
         } else
            this.entities = new CanonicalEntity[]{};
      }
      this.defentity = (CanonicalEntity) defentity.getEntity();
      this.indexed = indexed;
   }
   
   public CanonicalEntityField(String name, String label, Collection entities, CanonicalEntity defentity)
   {
      this.name = name;
      this.label = label;
      if (entities.size() == 0)
         this.entities = new CanonicalEntity[]{};
      else {
         Iterator i = entities.iterator();
         Object o = i.next();
         if (o instanceof CanonicalEntity)
            this.entities = (CanonicalEntity[]) (entities.toArray(new CanonicalEntity[] {}));
         else if (o instanceof CanonicalViewer) {
            List a = new Vector();
            a.add(((Viewer) o).getEntity());
            while (i.hasNext())
               a.add(((Viewer) i.next()).getEntity());
            this.entities = (CanonicalEntity[]) (a.toArray(new CanonicalEntity[] {}));
         } else
            this.entities = new CanonicalEntity[]{};
      }
      this.defentity = defentity;
   }

   public CanonicalEntityField(String name, String label, Collection entities, CanonicalViewer defentity)
   {
      this.name = name;
      this.label = label;
      if (entities.size() == 0)
         this.entities = new CanonicalEntity[]{};
      else {
         Iterator i = entities.iterator();
         Object o = i.next();
         if (o instanceof CanonicalEntity)
            this.entities = (CanonicalEntity[]) (entities.toArray(new CanonicalEntity[] {}));
         else if (o instanceof CanonicalViewer) {
            List a = new Vector();
            a.add(((Viewer) o).getEntity());
            while (i.hasNext())
               a.add(((Viewer) i.next()).getEntity());
            this.entities = (CanonicalEntity[]) (a.toArray(new CanonicalEntity[] {}));
         } else
            this.entities = new CanonicalEntity[]{};
      }
      if(defentity != null)
         this.defentity = (CanonicalEntity) defentity.getEntity();
      else
         this.defentity = null;
   }
   
   public CanonicalEntityField(String name, String label, CanonicalEntity[] entities, CanonicalEntity defentity)
   {
      this.name = name;
      this.label = label;
      this.entities = entities;
      this.defentity = defentity;
   }

   public CanonicalEntityField(String name, String label, CanonicalEntity[] entities, CanonicalViewer defentity)
   {
      this.name = name;
      this.label = label;
      this.entities = entities;
      this.defentity = (CanonicalEntity) defentity.getEntity();
   }
    
   public CanonicalEntityField(String name, String label, CanonicalViewer[] entities, CanonicalEntity defentity)
   {
      this.name = name;
      this.label = label;
      this.entities = new CanonicalEntity[entities.length];
      for (int i=0; i<entities.length; i++)
         this.entities[i] = (CanonicalEntity) entities[i].getEntity();
      this.defentity = defentity;
   }

   public CanonicalEntityField(String name, String label, CanonicalViewer[] entities, CanonicalViewer defentity)
   {
      this.name = name;
      this.label = label;
      this.entities = new CanonicalEntity[entities.length];
      for (int i=0; i<entities.length; i++)
         this.entities[i] = (CanonicalEntity) entities[i].getEntity();
      this.defentity = (CanonicalEntity) defentity.getEntity();
   }
   
   protected String print()
   {
      String s = "";
      s += "<select name='"+name+"'>\n";
      for (int i=0; i<entities.length; i++)
         try {
            String val;
            if (indexed) val = ""+i;
            else val = entities[i].toString();
            if (entities[i].equals(defentity))
               s += "   <option value='"+val+"' selected='selected'>"+Tools.escapeXML(entities[i].getPrintable())+"</option>\n";
            else
               s += "   <option value='"+val+"'>"+Tools.escapeXML(entities[i].getPrintable())+"</option>\n";
         } catch (AssassinSQLException ASQLe) {}
      s += "</select>\n";
      return s;
   }

   static CanonicalEntity getResultEntity(String POSTvalue) throws NotSerialisedEntityException
   {
      return CanonicalEntity.deSerialise(POSTvalue);
   }
 
   public static CanonicalViewer getResultViewer(String POSTvalue) throws NotSerialisedEntityException
   {
      return (CanonicalViewer) CanonicalEntity.deSerialise(POSTvalue).getViewer();
   }
   
}


