/* 
 * CUAG AutoUmpire System
 * 
 * Copyright (C) 2003/2004 Matthew Johnson, Adam Biltcliffe,
 * Michael Cripps, Martin O'Leary, Edward Allcutt and James Osborn
 *
 * This program is free software; you can redistribute it and/or 
 * modify it under the terms of the GNU General Public License 
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version. 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details. 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * To Contact the authors, please email assassins@srcf.ucam.org
 *
 */

package org.ucam.srcf.assassins;

/**
 * Class for state changes which affect plugin-specific state. All such
 * state changes are stored as a triple of plugin name, field and new value
 * along with a reference to the assassin to whom they apply.
 *
 * @author Matthew Johnson <autoumpire@matthew.ath.cx>
 */
public class PluginStateChange extends StateChange
{
  
   public PluginStateChange(Assassin a, Plugin p, String field, String value)
   {
      super();
      this.ass = a;
      this.pluginName = p.getPluginName();
      this.field = field;
      // to unambiguate Strings from CanonicalEntity serialisations
      if (value.startsWith("__::"))
         value = " " + value;
      this.value = value;
   }
   public PluginStateChange(Assassin a, Plugin p, String field, CanonicalEntity value)
   {
      super();
      this.ass = a;
      this.pluginName = p.getPluginName();
      this.field = field;
      this.value = value;
   }
   public PluginStateChange(Assassin a, Plugin p, String field, CanonicalViewer value)
   {
      super();
      this.ass = a;
      this.pluginName = p.getPluginName();
      this.field = field;
      this.value = value.getEntity();
   }
   public PluginStateChange(AssassinViewer a, Plugin p, String field, String value)
   {
      super();
      this.ass = (Assassin) a.getEntity();
      this.pluginName = p.getPluginName();
      this.field = field;
      // to unambiguate Strings from CanonicalEntity serialisations
      if (value.startsWith("__::"))
         value = " " + value;
      this.value = value;
   }
   public PluginStateChange(AssassinViewer a, Plugin p, String field, CanonicalEntity value)
   {
      super();
      this.ass = (Assassin) a.getEntity();
      this.pluginName = p.getPluginName();
      this.field = field;
      this.value = value;
   }
   public PluginStateChange(AssassinViewer a, Plugin p, String field, CanonicalViewer value)
   {
      super();
      this.ass = (Assassin) a.getEntity();
      this.pluginName = p.getPluginName();
      this.field = field;
      this.value = value.getEntity();
   }
   PluginStateChange(Integer scid)
   {
      super(scid);
   }
   PluginStateChange(int scid)
   {
      super(scid);
   }
   public String toString() { return "__::"+StateChange.class.getName()+"("+id+")"; }
}

