package org.ucam.srcf.assassins.strawberry;

/**
 * Interface for defining operations for Scorer class.
 */

abstract class Operator implements Comparable {
	protected static int precedence=0;
	protected static char character=0;
	
	/** Returns the precedence for this operation, higher is better */
	public static int getPrecedence() {
		return precedence;
	}

	/** Returns the character representing this operation.
	 * eg '+', '^', '*'
	 * 
	 * @return the value we return
	 */
	public static char getCharacter() {
		return character;
	}

	/**
	 * Compares two operations based upon the precedence.
	 * Higher is better.
	 * @return -ive if less, 0 if equal, +ive if greater
	 */
	public int compareTo(Object to) {
		int to_prec=((Operator) to).getPrecedence();
		return to_prec - this.precedence;
	}

	/** Conducts this operation.
	 * eg for + evaluate(3,4) returns 3+4=7
	 *
	 * @param lval the value to the left of this operation
	 * @param rval the value to the right of this operation
	 * @return The final value of this operation conducted on the obvious.
	 */
	abstract double evaluate(double lval, double rval);
}
