/**
 * Class for dealing with static scoring.
 */

package org.ucam.srcf.assassins.strawberry;

import java.util.Map;

import org.ucam.srcf.assassins.AssassinViewer;
import org.ucam.srcf.assassins.Death;
import org.ucam.srcf.assassins.exceptions.AssassinSQLException;

public interface StaticScoringAlgorithm {
   /**
    * Initilise algorithm. 
    * Will be called before any other functions.
    * @param n Maximum number of assassins the algorithm has to deal with
    */
   public void init (int n, Strawberry straw);

   /** 
    * Scores the deaths the algorithm knows about.
    * This is <i>no longer</i> the final call of the algorithm, and in
    * reality you will get intermixed calls of addDeath and score as
    * we iterate through the game's progress.
    * @return Map from AssassinViewer to Double representing score.
    */
   public Map score();

   /**
    * Tell the algorithm about a death.
    * May <i>no longer</i> assume score() hasn't been called yet.
    * @param d Death we're telling it about.
    */
   public void addDeath(Death d) throws AssassinSQLException;

   /**
    * Tells the algorithm about an innocent death(s).
    * It is up to the alogrithm to use it as it sees fit
    * @param who Who killed it/them?
    * @param count How many they killed.
    */
   public void reportInnocents(AssassinViewer who, int count) throws AssassinSQLException;

   /**
    * Tells the algorithm about bonuses.
    * It is up to the algorithm to actually apply them
    * @param who Who gets the bonus.
    * @param value How much is it worth?
    */
   public void giveBonus(AssassinViewer who, double value) throws AssassinSQLException;
}
