/**
 * Class to track changes in differnt variables for all players
 */

package org.ucam.srcf.assassins.strawberry;

import java.util.Map;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Set;

import org.ucam.srcf.assassins.AssassinViewer;
import org.ucam.srcf.assassins.Death;
import org.ucam.srcf.assassins.Debug;
import org.ucam.srcf.assassins.EventViewer;
import org.ucam.srcf.assassins.StateChange;
import org.ucam.srcf.assassins.Tools;

import org.ucam.srcf.assassins.exceptions.AssassinSQLException;

class VarTracker {
   Map state;

   Map scores;

   String pluginName;

   VarTracker(Set interested, String pname) throws AssassinSQLException {
      /* Generate an empty map */
      this.state = new HashMap();
      Iterator i = interested.iterator();
      while (i.hasNext())
         this.state.put(i.next(), new HashMap());
      /* Setup a new set of scores */
      refreshScores(null);
      /* Cache the plugin name which owns us */
      this.pluginName = pname;
   }

   public Map getState() {
      return state;
   }

   public void setProperty(AssassinViewer a, String what, String value) {
      Map aMap = (Map) state.get(a);
      aMap.put(what, value);
   }

   public String getProperty(AssassinViewer a, String what) {
      Map aMap = (Map) state.get(a);
      String ret_val= (String) aMap.get(what);
		if(ret_val==null)
			ret_val="";
		return ret_val;
   }

   public void setIntProperty(AssassinViewer a, String what, int value) {
      setProperty(a, what, String.valueOf(value));
   }

   /**
    * returns the property cast to an integer.
    * 
    * @param a
    *           The Assassin the property belongs to.
    * @param what
    *           The value to retrieve
    * @return The value in question, 0 if it doens't exist
    */
   public int getIntProperty(AssassinViewer a, String what) {
		if(Debug.debug) Debug.print(this, "Getting property \""+what+"\" which has value \""+getProperty(a, what)+"\"");
		if(getProperty(a, what)==null || getProperty(a, what).equals(""))
			return 0;
      return Integer.parseInt(getProperty(a, what));
   }

   /**
    * Increment the given property by a specified amount.
    * 
    * @param a
    *           The Assassin the Property belongs to.
    * @param what
    *           String specifying what the property is.
    * @param increment
    *           The amount to increment it by.
    * @return The new value
    */
   public int incIntProperty(AssassinViewer a, String what, int increment) {
      int initial = getIntProperty(a, what);
      initial += increment;
      setIntProperty(a, what, initial);
      return initial;
   }

   public Map refreshScores(EventViewer e) throws AssassinSQLException {
      if (e == null) return scores = null;

      scores = new HashMap();
      Map participants = e.getParticipants();
      Iterator k = participants.keySet().iterator();
      while (k.hasNext())
         scores.put(k.next(), null);

      return scores;
   }

   public void trackInnocent(AssassinViewer a) {
      incIntProperty(a, "Innoccents", 1);
   }

   public void trackDeath(Death sc) {
      AssassinViewer killer, victim;
      try {
         killer = sc.getKiller();
         victim = sc.getVictim();
      } catch (AssassinSQLException asqle) {
         return;
      }

      incIntProperty(killer, "Kills", 1);
      incIntProperty(victim, "Deaths", 1);
   }

   public boolean trackingAssassin(AssassinViewer a) {
      return (state.get(a) != null);
   }

   /**
    * Read in the values for an assassin at a given time.
    * Does tracking all at once in SQL for a quick start.
    * Uses time &lt; and not &lt;=.
    */
   public void readValuesForAssassinAtTime(Strawberry straw, AssassinViewer a, long t) throws AssassinSQLException {
      if(Debug.debug) Debug.print(this, "reading Values for "+a+" ("+a.getPlayerViewer().getName()+") at time "+Tools.prettyDateAndTime(t));
      if(state.get(a) == null)
         state.put(a, new HashMap());
      /* Get kills and deaths */
      setIntProperty(a, "Kills", a.countKillsAtTime(t));
      setIntProperty(a, "Deaths", a.countDeathsAtTime(t));
      /* Get scores */
      readIn(a, t, "BaseScore", straw.defaultStartingBaseScore);
      readIn(a, t, "PostGameMod", 0);
      readIn(a, t, "FinalScore", getIntProperty(a, "BaseScore") + 
            getIntProperty(a,"PostGameMod"));
      /* Get vital status */
      if(a.isAliveAtTime(t)) {
         if(Debug.debug) Debug.print(this, "Assassin alive at time "+Tools.prettyDateAndTime(t));
         setProperty(a, "alive", "true");
      } else {
         if(Debug.debug) Debug.print(this, "Assassin dead at time "+Tools.prettyDateAndTime(t));
         setProperty(a, "alive", "false");
      }
   }

   private void readIn(AssassinViewer a, long t, String field, int defaultVal) throws AssassinSQLException {
      List scs = StateChange.getFieldValueAtTime(a, pluginName, field, t);
      if(scs.isEmpty())
         setIntProperty(a, field, defaultVal);
      else {
         StateChange sc = (StateChange) scs.get(0);
         setIntProperty(a, field, Integer.parseInt((String) sc.getValue()));
      }
   }

}
