/* 
 * CUAG AutoUmpire System
 * 
 * Copyright (C) 2003/2004 Matthew Johnson, Adam Biltcliffe,
 * Michael Cripps, Martin O'Leary, Edward Allcutt and James Osborn
 *
 * This program is free software; you can redistribute it and/or 
 * modify it under the terms of the GNU General Public License 
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version. 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details. 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * To Contact the authors, please email assassins@srcf.ucam.org
 *
 */

package org.ucam.srcf.assassins;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.InputStreamReader;
import java.io.PrintWriter;
import java.io.FileNotFoundException;

import java.util.Arrays;
import java.util.Date;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Random;
import java.util.Vector;

import org.ucam.srcf.assassins.exceptions.AssassinSQLException;
import org.ucam.srcf.assassins.exceptions.NotSerialisedEntityException;
import org.ucam.srcf.assassins.xml.Config;

import cx.ath.matthew.cgi.CGI;
import cx.ath.matthew.cgi.CGIHeaderSentException;
import cx.ath.matthew.cgi.CGIInvalidContentFormatException;
import cx.ath.matthew.cgi.DisplayField;
import cx.ath.matthew.cgi.HiddenField;
import cx.ath.matthew.cgi.MultipleDropDown;
import cx.ath.matthew.cgi.Password;
import cx.ath.matthew.cgi.SubmitButton;
import cx.ath.matthew.cgi.TextArea;
import cx.ath.matthew.cgi.TextField;



/**
 * The main class that manages the CGI interface.
 */
public class umpirecgi  extends CGI
{
   public static final String authentication_file = "../config/authenticate.txt";
   public static String[] getColleges()
   {
      if (null == collegelist) {
         String colleges = Config.getConfig("vanilla", "colleges");
         if (null == colleges) collegelist = new String[0];
         else 
            collegelist = colleges.split(" *, *");
      }
      return collegelist;
   }
   private static String[] collegelist = null;
 
   public static final String[] waterstatuses = new String[] { "No Water", "Water With Care", "Full Water" };
   public static final String[] PLUGINS = new String[] { 
      "org.ucam.srcf.assassins.chocolate.Chocolate",
      "org.ucam.srcf.assassins.peach.Peach",
      "org.ucam.srcf.assassins.pistachio.Pistachio",
      "org.ucam.srcf.assassins.stracciatella.Stracciatella",
      "org.ucam.srcf.assassins.tuttifrutti.Tuttifrutti",
      "org.ucam.srcf.assassins.watermelon.Watermelon"
   };
	static Authentication currentAuth;
   private static String rootuser = null;
   public static String getRootUser()
   {
      return rootuser;
   }

   /** These are set after authentication
    */
   static boolean customnowtime = false;

   /* Path to root directory where css/ and img/ live */
   private static String basePath;

	/** Which tab are we on? */
	private String tab = null;
	/** What entity is selected? */
	private String selectedEntity = null;
   void setSelectedEntity(String selectedEntity) { this.selectedEntity = selectedEntity; }
	/** Command map in case we need to pull any out for niceness */
	private Map commands;
	/** What command string should we send back to the program if someone changes tabs? */

   /**
    * Just call the CGI stuff
    */
   public static void main(String[] args)
   {
      CGI cgi = new umpirecgi();
      cgi.doCGI(args);
   }
   public static void setupCommandMap(Map commands)
   {
      commands.put("show-plugins", new CGIShowPlugins()); 
      commands.put("set-time", new CGISetTime());
      commands.put("logout", new CGILogout());
      CGIAddPlayer cgiaddplayer = new CGIAddPlayer();
      commands.put("add-player", cgiaddplayer);
      commands.put("change-participant-pseudonym", new CGIChangeParticipant());
      commands.put("add-participant", new CGIAddParticipant()); 
      CGIAddAssassin cgiaddassassin = new CGIAddAssassin();
      commands.put("add-assassin", cgiaddassassin);
      CGIAddEvent cgiaddevent = new CGIAddEvent();
      commands.put("add-event", cgiaddevent);
      commands.put("edit-event", new CGIEditEvent());
      commands.put("edit-assassin", new CGIEditAssassin());
      commands.put("edit-report", new CGIEditReport()); 
      commands.put("remove-report", new CGIRemoveReport());
      CGIAddReport cgiaddreport = new CGIAddReport();
      commands.put("add-report", cgiaddreport);
      commands.put("edit-pseudonym", new CGIEditPseudonym());
      commands.put("add-pseudonym", new CGIAddPseudonym());
      commands.put("add-death", new CGIAddDeath());
      commands.put("add-event-with-report", 
            new CGIAddEventWithReport(cgiaddevent, cgiaddreport));
      commands.put("add-player-with-assassin", 
            new CGIAddPlayerWithAssassin(cgiaddplayer, cgiaddassassin));
      commands.put("dump-state", new CGIDumpState());
      commands.put("update-state", new CGIUpdateState());
      commands.put("generate-mail", new CGIGenerateMail()); 
      commands.put("generate-pages", new CGIGeneratePages());
      commands.put("page", new CGIDynamicPage());
      commands.put("report-kill", new CGIReportKill());
      commands.put("report-death", new CGIReportDeath());
      commands.put("backup", new CGIDbBackup());
      commands.put("quick", new CGIQuickLinks());
      commands.put("remove-statechange", new CGIRemoveStateChange());
   }

   public static boolean authIsAtLeast(AuthLevel a) {
      /* If no auth is set then its a CLI user */
      if(currentAuth==null)
         return true;
      return currentAuth.isAtLeast(a);
   }

   public static PlayerViewer authGetPlayerViewer() {
      if(currentAuth==null)
         return null;
      return currentAuth.getPlayerViewer();
   }
   
   void printmap(Map a) throws CGIInvalidContentFormatException
   {
      Iterator i = a.keySet().iterator();
      while (i.hasNext()) {
         Object k = i.next();
         Object v = a.get(k);
         out (k + " => " + v + " <br/>");
      }
   }

   /**
    * Checks for a valid authentication, or prompts for it if not.
    */
   boolean authenticate(Map POST, Map GET, Map ENV, Map COOKIE, String[] params) throws CGIHeaderSentException, AssassinSQLException, CGIInvalidContentFormatException
   {
      return authenticate(POST, GET, ENV, COOKIE, params, true);
   }
   boolean authenticate(Map POST, Map GET, Map ENV, Map COOKIE, String[] params, boolean prompt) throws CGIHeaderSentException, AssassinSQLException, CGIInvalidContentFormatException
   {
      if (Debug.debug) {
         Debug.print(this, "Authenticating with:");
			try {
	         Debug.print(this, "existing: "+currentAuth.getEmail()+"/"+currentAuth.getSecret());
			} catch (AssassinSQLException ex) {}
         Debug.print(this, "POST: "+POST.get("authuser")+"/"+POST.get("authpw")+" ("+POST.get("sessionID")+")");
         Debug.print(this, "COOKIE: "+COOKIE.get("sessionID"));
      }
    
      // check details in cookies
		try {
	      if (AuthLevel.PUBLIC != currentAuth.validateSession((String) COOKIE.get("sessionID"))) {
	         return true;
	      }
		} catch (AssassinSQLException e) {
			if(Debug.debug) Debug.print(this, "Failed to authenticate cookie, SQL error!\n"+e);
		}

		String authuser="";
		String authpw="";
      // check details in POST
      if (POST.get("authuser") != null && POST.get("authpw") != null) {
			authuser=(String) POST.get("authuser");
			authpw=(String) POST.get("authpw");
         POST.remove("authuser");
         POST.remove("authpw");
         if (POST.get("authlogin") != null) {
            POST.remove("submit");
            POST.remove("authlogin");
         }
         // TODO: check in here that they are valid. Individal functions should check if username is correct.
			try {
	         if (AuthLevel.PUBLIC != currentAuth.validateUser(authuser, authpw)) {
	            String path = (String) ENV.get("REQUEST_URI");
	            if (null != path) path = path.substring(0, path.lastIndexOf('/'));
	            // set cookie with 8hr timeout
					if(Debug.debug) Debug.print(this,"Setting session cookie with value "+currentAuth.getSessionID()+" with setcookie((\"sessionID\", "+currentAuth.getSessionID()+", "+path+", "+(String) ENV.get("SERVER_NAME")+", "+new Date(System.currentTimeMillis() + 28800000)+", true);");
					setcookie("sessionID", currentAuth.getSessionID(), path, (String) ENV.get("SERVER_NAME"), new Date(System.currentTimeMillis() + 28800000), true);
	            return true;
	         } else {
               // login falied and uname/password tried
               sendHTMLHeader("Log In Failed");
               out("<p>");
               out("   <b>Bad username and/or password.</b>");
               out("</p>");
               printLoginForm(POST, params, authuser); 
               sendHTMLFooter(false);
               return false;
            }
			} catch (AssassinSQLException e) {
				if(Debug.debug) Debug.print(this,"Failed to auth with password, SQL error!");
			}
      }
		
      if (currentAuth.isAtLeast(AuthLevel.LOGGED_IN)) return true;
      
      // show form
      if (prompt) {
         sendHTMLHeader("Log In");
         out("<p>");
         out("   Please log in.");
         out("</p>");
         printLoginForm(POST, params, authuser); 
         sendHTMLFooter(false);
         return false;
      }  
      else return false;
   }

   private void printLoginForm(Map POST, String[] params, String authuser) {
      String p = "";
      for (int i=0; i<params.length; i++)
         p += params[i]+",";
      HTMLForm form = new HTMLForm("umpire.cgi?"+p, "Log In");
      Iterator i = POST.keySet().iterator();
      while (i.hasNext()) {
         String name = (String) i.next();
         String value = (String) POST.get(name);
         form.addField(new HiddenField(name, value));
      }
      form.addField(new HiddenField("authlogin", "true"));
      form.addField(new TextField("authuser", "Email Address:", authuser));
      form.addField(new Password("authpw", "Password:", ""));
      form.display(this);
   }

	
	protected static void printLeftNavFunctions(umpirecgi cgi) throws AssassinSQLException, CGIInvalidContentFormatException {
		/* Do the tabs */
		cgi.out("<div id=\"nav_tabs\">");
		cgi.out("   <ul>");
		cgi.out("      <li id=\"Players\" class=\"unselected\"><a href=\"umpire.cgi?"/*resume=true&amp;*/+"tab=Players\">Players</a></li>");
		cgi.out("      <li id=\"Events\" class=\"unselected\"><a href=\"umpire.cgi?"/*resume=true&amp;*/+"tab=Events\">Events</a></li>");
		cgi.out("      <li id=\"Functions\" class=\"selected\">Functions</li>");
		cgi.out("   </ul>");
		cgi.out("</div>");
		cgi.out("<span class=\"navclear\"></span>");

		/* Do the function list */
		
		/* Build a section mapping */
		Iterator i = cgi.commands.keySet().iterator();
		Map cmdSecMap = new HashMap();
		Map currentSection = null;
		while(i.hasNext()) {
			String name=(String) i.next();
			CGICommand current = (CGICommand) cgi.commands.get(name);
         if (null == current.getSection()) continue;
			if (!currentAuth.isAtLeast(current.accessLevel())) continue;
			if((currentSection = (Map) cmdSecMap.get(current.getSection()))==null) {
				cmdSecMap.put(current.getSection(), (currentSection=new HashMap()));
			}
			currentSection.put(name, current);
		}

		/* print the list */
		Object a[] = cmdSecMap.keySet().toArray();
		Arrays.sort(a);
		cgi.out("<div class=\"list\">");
		cgi.out("   <ul>");
      cgi.out("      <li><b><a href=\"umpire.cgi\">Back to Command List</a></b></li>");
		for(int j=0; j<a.length; j++) {
			currentSection = (Map) cmdSecMap.get(a[j]);
			Object b[] = currentSection.keySet().toArray();
			Arrays.sort(b);
			cgi.out("      <li><b><a href='javascript:expand(\"_fn"+j+"\");' style='display:none;' class='button' id='expand_fn"+j+"'><img src='"+basePath+"img/plus.png' alt='+'/> "+a[j]+"</a><a href='javascript:hide(\"_fn"+j+"\");' class='button' id='hide_fn"+j+"'><img src='"+basePath+"img/minus.png' alt='-'/> "+a[j]+"</a></b>");
			cgi.out("         <ul class='hidden' id='list_fn"+j+"' style='display:block;'>");
			for(int k=0; k<b.length; k++) {
				CGICommand current = (CGICommand) currentSection.get(b[k]);
				cgi.out("            <li><a href=\"umpire.cgi?"+b[k]+"\" title=\""+current.getDescription()+"\">"+current.getName()+"</a></li>");
			}
			cgi.out("         </ul>");
			cgi.out("      </li>");
		}
      cgi.out("   </ul>");
		cgi.out("</div>");
	}

	protected static void printLeftNavPlayers(umpirecgi cgi) throws AssassinSQLException, CGIInvalidContentFormatException {
		/* Do the tabs */
		cgi.out("<div id=\"nav_tabs\">");
		cgi.out("   <ul>");
		cgi.out("      <li id=\"Players\" class=\"selected\">Players</li>");
		cgi.out("      <li id=\"Events\" class=\"unselected\"><a href=\"umpire.cgi?"/*resume=true&amp;*/+"tab=Events\">Events</a></li>");
		cgi.out("      <li id=\"Functions\" class=\"unselected\"><a href=\"umpire.cgi?"/*resume=true&amp;*/+"tab=Functions\">Functions</a></li>");
		cgi.out("   </ul>");
		cgi.out("</div>");
		cgi.out("<span class=\"navclear\"></span>");

		cgi.out("<div class=\"list\">");
		cgi.out("   <ul>");
		
		/* Player/Assassin related fns */
      Object[] cmds = cgi.commands.keySet().toArray();
      Arrays.sort(cmds);
      for(int i=0; i<cmds.length; i++) {
         String name = (String) cmds[i];
         
         /* Only want Add commands */
         if(!name.substring(0,3).equals("add"))
            continue;
         
         CGICommand current = (CGICommand) cgi.commands.get(name);
         
         if(!"Assassin".equals(current.getSection()))
            continue;
         
         if (currentAuth.isAtLeast(current.accessLevel())) {
            cgi.out("      <li class=\"nonSelAss\"><b>");
            CGITools.printlink(cgi, name, "umpire.cgi?tab=Players&"+name, current.getName());
            cgi.out("      </b></li>");
         }
      }

      char selectedLetter = '_';
      try {
         Object selectedObject = CanonicalEntity.deSerialise(cgi.selectedEntity);
         if(selectedObject instanceof Assassin)
            selectedLetter = ((Assassin) selectedObject).getPlayer().getName().toUpperCase().charAt(0);
      } catch (NotSerialisedEntityException nsee) {}
		
		/* Do the player list */
		List assassins = Assassin.getAllAssassinsOrderByRealName();
	   Iterator i=assassins.iterator();
		char currentLetter=0;
		while(i.hasNext()) {
			Assassin current = (Assassin) i.next();
         if("".equals(current.getPlayer().getName())) continue;
			if(currentLetter != current.getPlayer().getName().toUpperCase().charAt(0)) {
				if(currentLetter!=0) {
               cgi.out("      </ul>");
               cgi.out("   </li>");
				}
				currentLetter = current.getPlayer().getName().toUpperCase().charAt(0);
            if(selectedLetter != currentLetter) {
               cgi.out("      <li><b><a href='javascript:expand(\""+currentLetter+"\");' id='expand"+currentLetter+"' class='button'><img src='"+basePath+"img/plus.png' alt='+' /> "+currentLetter+"</a><a href='javascript:hide(\""+currentLetter+"\");' style='display:none;' id='hide"+currentLetter+"' class='button'><img src='"+basePath+"img/minus.png' alt='-' /> "+currentLetter+"</a></b>");
               cgi.out("         <ul class='hidden' id='list"+currentLetter+"'>");
            } else {
               cgi.out("      <li><b><a href='javascript:expand(\""+currentLetter+"\");' style='display:none;' id='expand"+currentLetter+"' class='button'><img src='"+basePath+"img/plus.png' alt='+' /> "+currentLetter+"</a><a href='javascript:hide(\""+currentLetter+"\");' id='hide"+currentLetter+"' class='button'><img src='"+basePath+"img/minus.png' alt='-' /> "+currentLetter+"</a></b>");
               cgi.out("         <ul class='hidden' id='list"+currentLetter+"' style='display:block;'>");
            }
			}
			String val=current.toString();
			if(!val.equals(cgi.selectedEntity)) {
            cgi.out("            <li class=\"nonSelAss\"><a href=\"umpire.cgi?tab=Players&selected="+val+"&edit-assassin&assassin="+val+"\">"+Tools.escapeXML(current.getPlayer().getName())+"("+current.getAid()+")</a></li>");
			} else {
            cgi.out("            <li class=\"SelAss\">"+Tools.escapeXML(current.getPlayer().getName())+"("+current.getAid()+")</li>");
			}
		}
      cgi.out("         </ul>");
      cgi.out("      </li>");
      cgi.out("   </ul>");
      cgi.out("</div>");
	}
	
	protected static void printLeftNavEvents(umpirecgi cgi) throws AssassinSQLException, CGIInvalidContentFormatException
   {
		/* Do the tabs */
		cgi.out("   <div id=\"nav_tabs\">");
		cgi.out("      <ul>");
		cgi.out("         <li id=\"Players\" class=\"unselected\"><a href=\"umpire.cgi?"/*resume=true&amp;*/+"tab=Players\">Players</a></li>");
		cgi.out("         <li id=\"Events\" class=\"selected\">Events</li>");
		cgi.out("         <li id=\"Functions\" class=\"unselected\"><a href=\"umpire.cgi?"/*resume=true&amp;*/+"tab=Functions\">Functions</a></li>");
		cgi.out("      </ul>");
		cgi.out("   </div>");
		cgi.out("   <span class=\"navclear\"></span>");

		cgi.out("<div class=\"list\">");
		cgi.out("   <ul>");

		/* Event related add fns */
      Object[] cmds = cgi.commands.keySet().toArray();
      Arrays.sort(cmds);
      for(int i=0; i<cmds.length; i++) {
         String name = (String) cmds[i];
         
         /* Only want Add commands */
         if(!name.substring(0,3).equals("add"))
            continue;
         
         CGICommand current = (CGICommand) cgi.commands.get(name);
         
         if(!"Event".equals(current.getSection()))
            continue;
         
         if (currentAuth.isAtLeast(current.accessLevel())) {
            cgi.out("      <li class=\"nonSelAss\"><b>");
            CGITools.printlink(cgi, name, "umpire.cgi?tab=Players&"+name, current.getName());
            cgi.out("      </b></li>");
         }
      }
		
		/* Do the event list, list by day and then by event on a sublist */
		String lastDay="__NO_PREV_DAY__";
		List events = Event.getAllEventsWithHeadline();
		Iterator i=events.iterator();
		int dayn=0;
		while(i.hasNext()) {
			Event current = (Event) i.next();
			if(!lastDay.equals(Tools.prettyDate(current.getEventTime()))) {
				if(!lastDay.equals("__NO_PREV_DAY__")) {
					cgi.out("            </ul>");
					cgi.out("         </li>");
				}
				cgi.out("         <li><b><a href='javascript:expand(\"_d"+dayn+"\");' id='expand_d"+dayn+"' class='button'><img src='"+basePath+"img/plus.png' alt='+' /> "+Tools.prettyDate(current.getEventTime())+"</a><a href='javascript:hide(\"_d"+dayn+"\");' id='hide_d"+dayn+"' style='display:none;' class='button'><img src='"+basePath+"img/minus.png' alt='-' /> "+Tools.prettyDate(current.getEventTime())+"</a></b>");
				cgi.out("            <ul class='hidden' id='list_d"+dayn+"'>");
				lastDay=Tools.prettyDate(current.getEventTime());
				dayn++;
			}
			String val=current.toString();
			if(!val.equals(cgi.selectedEntity)) {
				cgi.out("               <li class=\"nonSelEvent\"><a href=\"umpire.cgi?tab=Events&selected="+val+"&edit-event&event="+val+"\">");
			} else {
				cgi.out("               <li class=\"SelEvent\">");
			}
			String event_str="";
			String event_fin="";
			if(!current.isActive()) {
				event_str+="<span class=\"inactive\">";
				event_fin+="</span>";
			}
			if(!current.isVisible()) {
				event_str+="<span class=\"invisible\">";
				event_fin+="</span>";
			}
			cgi.out("                  "+event_str+Tools.escapeXML(current.getPrintable())+event_fin);
			if(!val.equals(cgi.selectedEntity)) {
				cgi.out("               </a></li>");
			} else {
				cgi.out("               </li>");
			}

		}
		cgi.out("            </ul>");
		cgi.out("         </li>");
		cgi.out("      </ul>");
		cgi.out("   </div>");
	}
	
	protected static void printLeftNav(umpirecgi cgi) throws AssassinSQLException, CGIInvalidContentFormatException {
		cgi.out("<div id='nav'>");

		if(cgi.tab==null) {
			printLeftNavFunctions(cgi);
		} else {
			if(cgi.tab.equals("Players")) {
				printLeftNavPlayers(cgi);
			} else if(cgi.tab.equals("Events")) {
				printLeftNavEvents(cgi);
			} else {
				/* Functions */
				printLeftNavFunctions(cgi);
			}
		}

		cgi.out("</div>");
	}
	
	protected static void printCommandList(CGI cgi, Map commands, boolean desciptions) throws CGIInvalidContentFormatException
   {
		cgi.out("<ul class='commandlist'>");
		Object[] a = commands.keySet().toArray();
		Arrays.sort(a);
		for (int j = 0; j < a.length; j++) {
			String command = (String) a[j];
			CGICommand com = (CGICommand) commands.get(command);
			if (!com.isVisible() || !currentAuth.isAtLeast(com.accessLevel())) continue;
			cgi.out("   <li>");
			CGITools.printlink(cgi, command, "umpire.cgi?"+command, com.getName());
			if(desciptions) {
				cgi.out("      - "+com.getDescription());
			}
			cgi.out("   </li>");
      }
      cgi.out("</ul>");
	}

   private void doSetupGame(Map POST, String[] params) throws Exception
   {
      if (params.length > 0 && params[0].equals("setupgame")) {

         // check auth code
         String authcode;
         try {
            FileInputStream in = new FileInputStream(authentication_file);
            BufferedReader inp = new BufferedReader(new InputStreamReader(in));
            authcode = inp.readLine();
            inp.close();
         } catch (FileNotFoundException fnf) {
            sendHTMLHeader("File Not Found");
            out("<h1>Authentication code file not found.</h1>");
            out("<p>Please ensure that the file <i>\"" + authentication_file + "\"</i> exists</p>");
            sendHTMLFooter(false);
            return;
         }

         if (authcode.equals(POST.get("authcode"))) {
            sendHTMLHeader("New Game Setup");
            if ("Done".equals(POST.get("submit"))) {
               // create game cgi directory and write out config

               // write config file

               File f = new File(Tools.findConfig());
               PrintWriter configfile = new PrintWriter(new FileOutputStream(f));
               configfile.println("<?xml version=\"1.0\" standalone=\"no\" ?>");
               configfile.println("<!DOCTYPE game SYSTEM \""+POST.get("xml-files")+"/config.dtd\">");
               configfile.println("<!-- You Just Lost The ... -->");
               configfile.println("<game>");
               configfile.println("   <plugin name=\"vanilla\" class=\"org.ucam.srcf.assassins.Vanilla\">");
               configfile.println("      <property name=\"game_prefix\">"+POST.get("game_prefix")+"</property>");
               configfile.println("      <property name=\"dbtype\">"+POST.get("dbtype")+"</property>");
               configfile.println("      <property name=\"dbhost\">"+POST.get("dbhost")+"</property>");
               configfile.println("      <property name=\"dbport\">"+POST.get("dbport")+"</property>");
               configfile.println("      <property name=\"dbname\">"+POST.get("dbname")+"</property>");
               configfile.println("      <property name=\"dbuser\">"+POST.get("dbuser")+"</property>");
               configfile.println("      <property name=\"dbpass\">"+POST.get("dbpass")+"</property>");
               configfile.println("      <property name=\"from-address\">"+POST.get("from-address")+"</property>");
               configfile.println("      <property name=\"game-start-date\">"+POST.get("game-start-date")+"</property>");
               configfile.println("      <property name=\"default-headline\">"+POST.get("default-headline")+"</property>");
               configfile.println("      <property name=\"path-to-pages\">"+POST.get("path-to-pages")+"</property>");
               configfile.println("      <property name=\"news-page-name\">"+POST.get("news-page-name")+"</property>");
               configfile.println("      <property name=\"updates-page-name\">"+POST.get("updates-page-name")+"</property>");
               configfile.println("      <property name=\"headlines-page-name\">"+POST.get("headlines-page-name")+"</property>");
               configfile.println("      <property name=\"xsl-file\">"+POST.get("xml-files")+"/webpage.xsl</property>");
               configfile.println("      <property name=\"admin-xsl-file\">"+POST.get("xml-files")+"/adminpage.xsl</property>");
               configfile.println("      <property name=\"root-address\">"+POST.get("root-address")+"</property>");
               configfile.println("      <property name=\"email-subject\">"+POST.get("email-subject")+"</property>");
               configfile.println("      <property name=\"cgiurl\">"+POST.get("cgiurl")+"</property>");
               Object o = POST.get("cols");
               if (o != null) {
                  if (o instanceof String) {
                     Vector v = new Vector();
                     v.add(o);
                     o = v;
                  }
                  Iterator i = ((List) o).iterator();
                  while (i.hasNext()) {
                     String colname = (String) i.next();
                     configfile.println("      <property name='colourcount-"+colname+"'>"+
                           POST.get("col."+colname+".count")+"</property>");
                     configfile.println("      <colourfilter name='"+colname+"'>");
                     configfile.println(POST.get("col."+colname+".filter"));
                     configfile.println("      </colourfilter>");
                  }
               }
               o = POST.get("templates");
               if (o != null) {
                  if (o instanceof String) {
                     Vector v = new Vector();
                     v.add(o);
                     o = v;
                  }
                  Iterator i = ((List) o).iterator();
                  int j = 1;
                  while (i.hasNext()) {
                     String tname = (String) i.next();
                     configfile.println("      <property name='template-"+j+"'>"+
                           tname+"</property>");
                     configfile.println("      <property name='template-page-name-"+j+"'>"+
                           POST.get("template."+tname+".output")+"</property>");
                     j++;
                  }
               }
               o = POST.get("newsdates");
               if (o != null) {
                  if (o instanceof String) {
                     Vector v = new Vector();
                     v.add(o);
                     o = v;
                  }
                  Iterator i = ((List) o).iterator();
                  long[] la = new long[((List) o).size()];
                  int j = 0;
                  while (i.hasNext())  {
                     String tstr = (String) i.next();
                     la[j++] = Tools.unFormatTime(tstr);
                  }
                  Arrays.sort(la);
                  for (j--; j >= 0; j--) {
                     configfile.println("      <property name='news-date-"+(j+1)+"'>"+
                           Tools.formatTime(la[j])+"</property>");
                  }
               }
               configfile.println("   </plugin>");
               o = POST.get("plugins");
               if (o != null) {
                  if (o instanceof String) {
                     Vector v = new Vector();
                     v.add(o);
                     o = v;
                  }
                  Iterator i = ((List) o).iterator();
                  while (i.hasNext()) {
                     String plc = (String) i.next();
                     try {
                        Class c = Class.forName(plc);
                        configfile.println("   <plugin name='"+
                              POST.get(plc+".name")+"' class='"+plc+"'>");
                        List l = (List) 
                           c.getDeclaredMethod("getConfigProperties",
                                 new Class[] {}).invoke(null,new Object[] {});
                                 Iterator j = l.iterator();
                                 while (j.hasNext()) {
                                    ConfigProperty cp = (ConfigProperty) j.next();
                                    configfile.println("      <property name='"+
                                          cp.getName()+"'>"+
                                          Tools.escapeXML((String) POST.get(
                                                c.getName()+"."+cp.getName()))+
                                          "</property>");
                                 }
                                 configfile.println("   </plugin>");
                     } catch (ClassNotFoundException e) {
                        out("<p>Error in adding plugin ("+plc+") (not found).</p>");
                     } catch (Exception e) {
                        configfile.println("   </plugin>");
                        out("<p>Error in adding plugin ("+plc+") (getting details).</p>");
                     }
                  }
               }
               configfile.println("</game>");
               configfile.close();

               Tools.exec( new String[] { "chmod", "660", "../config/config.xml" } );

               // initialise database
               Config.init("../config/config.xml");
               DatabaseBackend.init();
               DatabaseBackend.initTables();

               // add umpire user
               Player umpire = new Player();
               umpire.setPassword((String) POST.get("root-pw"));
               umpire.setName((String) POST.get("umpire-name"));
               umpire.setCollege((String) POST.get("umpire-college"));
               umpire.setEmail((String) POST.get("root-address"));
               umpire.setAddress((String) POST.get("umpire-address"));
               /* NOT CREATING ASSASSIN FOR THE UMPIRE
                  Assassin umpass = new Assassin();
                  umpire.addAssassin(umpass);
                  Pseudonym umpps = new Pseudonym();
                  umpps.setValue("The Umpire");
                  Event e = new Event();
                  e.setActive(true);
                  e.setEventTime(Tools.getNowTime());
                  e.addStateChange(new PseudonymGain(umpass, umpps));
                  e.addStateChange(new AssassinActivation(umpass)); */

               DatabaseBackend.commitAll();

               out("<p>");
               out("   The game has been created. Please log in ");
               CGITools.printlink(this, "login", "umpire.cgi", "here");
               out("   .");
               out("</p>");
            }
            else {
               HTMLForm form = new HTMLForm("umpire.cgi?setupgame", "Done");
               CGITools.posttoform(POST, form, new String[] { "newplugins", "newcol", "newcolcount",
                  "newcolfilter","newtemplatefile", "newtemplatename", "newnewsdate" });
               if ("Add Plugin".equals(POST.get("submit"))) {
                  Object o = POST.get("newplugins");
                  if (o != null) {
                     if (o instanceof String) { // one new plugin
                        Vector v = new Vector();
                        v.add(o);
                        o = v;
                     }
                     if (o instanceof List) { // several new plugins
                        Iterator i = ((List) o).iterator();
                        while (i.hasNext()) {
                           String pl = (String) i.next();
                           if ("".equals(pl)) continue;

                           Class c = Class.forName(pl);
                           form.addField(new HiddenField("plugins", pl));

                           try {
                              form.addField(new DisplayField("<b>"+c.getName()+"</b>", 
                                       "<b>"+(String)
                                       c.getDeclaredMethod("getDescription", 
                                          new Class[] {}).invoke(null, 
                                             new Object[] {})+"</b>"));
                           } catch (Exception e) {}

                           String pln = pl.substring(pl.lastIndexOf('.')+1);
                           form.addField(new TextField(pl+".name", 
                                    "Plugin Name:", pln));

                           try {
                              List l = (List) 
                                 c.getDeclaredMethod("getConfigProperties",
                                       new Class[] {}).invoke(null,new Object[] {});
                                       Iterator j = l.iterator();
                                       while (j.hasNext()) {
                                          ConfigProperty cp = (ConfigProperty) j.next();
                                          form.addField(new TextField(
                                                   c.getName()+"."+cp.getName(), 
                                                   cp.getDescription(), 
                                                   cp.getDefault()));
                                       }
                           } catch (Exception e) {}

                        }
                     }
                  }
               }
               else if ("Add Colour Type".equals(POST.get("submit"))) {
                  String colname = (String) POST.get("newcol");
                  form.addField(new DisplayField("<b>Colour Type Added</b>", 
                           colname));
                  form.addField(new HiddenField("cols", colname));
                  form.addField(new HiddenField("col."+colname+".count", 
                           (String) POST.get("newcolcount")));
                  form.addField(new HiddenField("col."+colname+".filter", 
                           (String) POST.get("newcolfilter")));
               }
               else if ("Add Template".equals(POST.get("submit"))) {
                  String newtemplate = (String) POST.get("newtemplatefile");
                  form.addField(new DisplayField("<b>Template Added</b>", 
                           newtemplate));
                  form.addField(new HiddenField("templates", newtemplate));
                  form.addField(new HiddenField("template."+newtemplate+".output",
                           (String) POST.get("newtemplatename")));
               }
               else if ("Add News Page".equals(POST.get("submit"))) {
                  form.addField(new DisplayField("<b>News Date Added</b>", 
                           (String) POST.get("newnewsdate")));
                  form.addField(new HiddenField("newsdates", 
                           (String) POST.get("newnewsdate")));
               }
               form.addField(new MultipleDropDown("newplugins", 
                        "Select Plugins to Add", PLUGINS, null, false));
               form.addField(new TextField("newplugins", "Add Other Plugin", ""));
               form.addField(new SubmitButton("submit", "Add Plugin"));
               form.addField(new TextField("newcol", "Add Colour Type", ""));
               form.addField(new TextField("newcolcount", "Number of colours", ""));
               form.addField(new TextArea("newcolfilter", "Colour Filter", "<and>\n	<filter plugin='' property='' condition='' />\n</and>"));
               form.addField(new SubmitButton("submit", "Add Colour Type"));
               form.addField(new TextField("newtemplatefile", "Template input file", ""));
               form.addField(new TextField("newtemplatename", "Template output page name", ""));
               form.addField(new SubmitButton("submit", "Add Template"));
               form.addField(new TextField("newnewsdate", "Start date of news page", ""));
               form.addField(new SubmitButton("submit", "Add News Page"));
               form.display(this);
            }
         }
         else {
            sendHTMLHeader("Invalid Auth Code");
            out("<p>");
            out("   You have not entered the correct authentication code from "+authentication_file+". You must copy this in exactly to authenticate as the Umpire.");
            out("</p>");
         }
         sendHTMLFooter(false);
      } else {
         // ask for details of game
         sendHTMLHeader("New Game Setup");
         out("<p>");
         out("   This is the setup system for the Assassins' Guild AutoUmpire System.");
         out("   You will have to enter the authentication code from "+authentication_file+" to create the game.");
         out("</p>");
         PrintWriter authfile;
         try {
            authfile = new PrintWriter(new FileOutputStream(authentication_file));
         } catch (FileNotFoundException ex) {
            out("<h1>Authentication code directory not found.</h1>");
            out("<p>Please ensure that the directory in which I shall create <i>\"" + authentication_file + "\"</i> exists</p>");
            sendHTMLFooter(false);
            return;
         }
         Random r = new Random(System.currentTimeMillis());
         byte[] authbytes = new byte[20];
         r.nextBytes(authbytes);
         String authstr = Tools.toHexStr(authbytes);
         authfile.println(authstr);
         authfile.close();
         HTMLForm form = new HTMLForm("umpire.cgi?setupgame", "Setup");
         form.addField(new TextField("authcode","Authentication Code",""));
         form.addField(new TextField("game_prefix","Game Prefix (e.g. 2004lent)",""));
         form.addField(new TextField("dbtype","Database Type","mysql"));
         form.addField(new TextField("dbhost","Database Host","localhost"));
         form.addField(new TextField("dbport","Database Port","3306"));
         form.addField(new TextField("dbname","Database Name","assassins"));
         form.addField(new TextField("dbuser","Database User","assassins"));
         form.addField(new Password("dbpass","Database Password",""));
         form.addField(new TextField("from-address","Send Emails From","assassins@srcf.ucam.org"));
         form.addField(new TextField("email-subject","Default Email Subject","The Assassins Guild"));
         form.addField(new TextField("game-start-date","Game Start (25-01-2004 05:00:00)",""));
         form.addField(new TextField("default-headline","Default Headline","[P1] killed [P2] ([L2])"));
         form.addField(new TextField("path-to-pages","Location of News Pages",""));
         form.addField(new TextField("cgiurl","URL of CGI Script","/assassins/umpire.cgi"));
         form.addField(new TextField("news-page-name","News Page Stem","news"));
         form.addField(new TextField("updates-page-name","Updates Page Stem","hups"));
         form.addField(new TextField("headlines-page-name","Headlines Page Stem","head"));
         form.addField(new TextField("xml-files","Location of XML files","xml"));
         form.addField(new TextField("umpire-name","Umpire's Name",""));
         form.addField(new TextField("umpire-college","Umpire's College",""));
         form.addField(new TextField("umpire-address","Umpire's Address",""));
         form.addField(new TextField("root-address","Umpire's Email Address",""));
         form.addField(new Password("root-pw","Umpire's Password",""));
         form.display(this);
         sendHTMLFooter(false);
      }
   }

   private void setTime(Map POST, Map GET) throws Exception
   {
      if (currentAuth.isAtLeast(AuthLevel.UMPIRE)) {
         if (POST.get("resettime") != null) {
            if (Debug.debug) Debug.print(this, "Resetting Time");
            Tools.setNowTime(new Date().getTime());
            currentAuth.unsetData("nowtime");
         } else if (POST.get("nowtime") != null) {
            if (Debug.debug) Debug.print(this, "Setting Custom Time "+POST.get("nowtime"));
            customnowtime = true;
            Tools.setNowTime(Tools.unFormatTime((String) POST.get("nowtime")));
            currentAuth.storeData("nowtime", ""+Tools.getNowTime());
         } else if (GET.get("nowtime") != null) {
            if (Debug.debug) Debug.print(this, "Setting Custom Time "+GET.get("nowtime"));
            customnowtime = true;
            Tools.setNowTime(Tools.unFormatTime((String) GET.get("nowtime")));
            currentAuth.storeData("nowtime", ""+Tools.getNowTime());
         } else if (currentAuth.getData("nowtime") != null) {
            Tools.setNowTime(Long.parseLong(currentAuth.getData("nowtime")));
            customnowtime = true;
         } else
            Tools.setNowTime(new Date().getTime());
      } else {
         Tools.setNowTime(new Date().getTime());
         currentAuth.unsetData("nowtime");
      }
   }

   /**
    * All the top level stuff in here.
    *
    * Note, atm, ENV and GET are empty.
    */
   protected void cgi(Map POST, Map GET, Map ENV, Map COOKIE, String[] params) throws Exception
   {
      if (Debug.debug) Debug.setOutput("error.log");
      if (Debug.debug) Debug.print((Class) null, "POST Map");
      if (Debug.debug) Debug.printMap((Class) null, POST);
      header("Content-Type", "text/html; charset=UTF-8");
      // try and find a config.xml
      // if it isn't there, run the game setup
      File f = new File(Tools.findConfig());
      if (!f.exists()) 
         doSetupGame(POST, params);

      // otherwise check the auth tokens
      // check for a parameter
      // no parameter => show command list
      // parameter => do command
      else {
         commands = new HashMap();
               
         /*
          * Init config file
          */
         Config.init("../config/config.xml");
			Authentication.readConfigValues();
			umpirecgi.currentAuth=Authentication.currentAuth;

         /*
          * Init database connection
          */
         DatabaseBackend.init();

         // initialise debug to use user speciied file instead 
         // of error.log
         Debug.setOutput();

         /*
          * Instantiate and init plugins
          */
         Map pluginnames = Config.getPluginList();
         if (Debug.debug) Debug.print(this, "Got "+pluginnames.size()+" plugins");
         Iterator i = pluginnames.keySet().iterator();
         Map plugins = new HashMap();

            while (i.hasNext())
            {
               String pname = (String) i.next();
               String classname = (String) pluginnames.get(pname);
               if (null != classname && !("".equals(classname)))
               {
                  if (Debug.debug) Debug.print(this, "instantiating " + pname);
                  Class c = Class.forName(classname);
                  Plugin p = (Plugin) c.newInstance();
                  if (Debug.debug) Debug.print(this, "setting name");
                  p.setPluginName(pname);
                  if (Debug.debug) Debug.print(this, "setting connector");
                  p.setPluginConnector(new PluginConnector());
                  if (Debug.debug) Debug.print(this, "getting new commands");
                  Map m = p.getCGICommands();                     
                  if (Debug.debug) Debug.printMap(umpirecgi.class, m);
                  if (Debug.debug) Debug.print(this, "Got "+m);
                  commands.putAll(m);
                  plugins.put(pname, p);
               }
            }

         setupCommandMap(commands);

         

         setTime(POST, GET);
         

         if (Debug.debug) Debug.print(this, "this: '"+currentAuth.getEmail()+"' root: '"+Authentication.getRootUID()+"'");

         if (params.length == 0 && GET.size() != 0) {
            Iterator j = GET.keySet().iterator();
            while (j.hasNext()) {
               String k = (String) j.next();
               if (!"".equals(k) && "".equals(GET.get(k)))
                  params = new String[] { k };
            }
         }

			/* Sort out the tab we're on and the selected entity */
			String value;
			if((value=(String) POST.get("tab")) == null) {
				value=(String) GET.get("tab");
			}
			if(value!=null) {
				this.setTab(value);
			} else {
				this.tab=currentAuth.getData("tab");
			}
			if((value=(String) POST.get("selected")) == null) {
				value=(String) GET.get("selected");
			}
			if(value!=null) {
				this.setEntity(value);
			} else {
				this.selectedEntity=currentAuth.getData("selected");
			}
         /*if (0 == params.length && 
             null != GET.get("resume") && 
             null != currentAuth.getData("command"))
            params = new String[] { currentAuth.getData("command") };*/
         String defplayercommand = Config.getConfig("vanilla", "default-player-command");
         String defumpirecommand = Config.getConfig("vanilla", "default-umpire-command");
         if (0 == params.length) {
            if (currentAuth.isAtLeast(AuthLevel.UMPIRE) && null != defumpirecommand)
               params = new String[] { defumpirecommand };
            else if (null != defplayercommand)
               params = new String[] { defplayercommand };
         }
         
         if (0 == params.length) {
            if (!authenticate(POST, GET, ENV, COOKIE, params)) return;
            currentAuth.unsetData("command");
            sendHTMLHeader("Select Command");
				out("<div id=\"main\">");
            out("<p>");
            out("   Select a command from the list below:");
            out("</p>");
            out("<ul class='commandlist'>");
            Object[] a = commands.keySet().toArray();
            Arrays.sort(a);
            for (int j = 0; j < a.length; j++) {
               String command = (String) a[j];
               CGICommand com = (CGICommand) commands.get(command);
               if (!currentAuth.isAtLeast(com.accessLevel())) continue;
               out("   <li>");
               CGITools.printlink(this, command, "umpire.cgi?"+command, com.getName());
               out("      - "+com.getDescription());
               out("   </li>");
            }
            out("</ul>");
				out("</div>");
            sendHTMLFooter(true);
         }
         else {
				if(Debug.debug) Debug.print(this, "Looking for command "+params[0]);
            CGICommand com = (CGICommand) commands.get(params[0]);
            
            if (com.accessLevel() == AuthLevel.PUBLIC) authenticate(POST, GET, ENV, COOKIE, params, false); 
            else if (!authenticate(POST, GET, ENV, COOKIE, params)) return; 
            
            if (null == com || !currentAuth.isAtLeast(com.accessLevel())) {
               sendHTMLHeader("Command Not Found");
               out("<p>");
               out("   You have specified an invalid command ("+params[0]+")");
               out("</p>");
               sendHTMLFooter(false);
            } else {
               if (Debug.debug) Debug.print(this, "Storing Command "+com);
               currentAuth.storeData("command", params[0]);
               sendHTMLHeader(params[0]);
					out("<div id=\"main\">");
               if (Debug.debug) Debug.print(this, "Running Command "+com);
               Map commandParams = new HashMap();
               if (null != GET) commandParams.putAll(GET);
               if (null != POST) commandParams.putAll(POST);
               if (null != ENV) commandParams.putAll(ENV);
               commandParams = CGITools.translateEntities(commandParams);
               for (int j=1; j<params.length; j++)
                  commandParams.put(""+j, params[j]);
               com.command(this, plugins, commandParams, Tools.getNowTime());
               DatabaseBackend.commitAll();
					out("</div>");
               sendHTMLFooter(true);
            }
         }
      }
   }

	protected void setTab(String value) throws AssassinSQLException {
		this.tab=value;
		currentAuth.storeData("tab", value);
	}
	
	protected void setEntity(String value) throws AssassinSQLException {
		this.selectedEntity=value;
		currentAuth.storeData("selected", value);
	}
	
	void printJavaScript() throws CGIInvalidContentFormatException {
      out("         <script type='text/javascript'>");
      out("         <!--");
      out("            function expand(letter)");
      out("            {");
      out("               document.getElementById('expand'+letter).style.display = 'none';");
      out("               document.getElementById('list'+letter).style.display = 'block';");
      out("               document.getElementById('hide'+letter).style.display = 'block';");
      /* The following line fixes the focus bug on gecko but has been 
       * known to minimise IE for no apparent reason whatsoever */
//      out("               document.getElementById('expand'+letter).blur();");
      out("            }");
      out("            function hide(letter)");
      out("          {");
      out("               document.getElementById('expand'+letter).style.display = 'block';");
      out("               document.getElementById('list'+letter).style.display = 'none';");
      out("               document.getElementById('hide'+letter).style.display = 'none';");
      /* The following line fixes the focus bug on gecko but has been 
       * known to minimise IE for no apparent reason whatsoever */
//      out("               document.getElementById('hide'+letter).blur();");
      out("            }");
      out("            -->");
      out("         </script>");
	}

   void sendHTMLHeader(String title) throws CGIInvalidContentFormatException
   {
		String socname = Config.getConfig("vanilla","society-name");
	   basePath=null;
      if (Config.isInited()) {
		  basePath = Config.getConfig("vanilla","base-path");
      }
      if(basePath == null)
         basePath="../";
      out("<?xml version=\"1.0\" encoding=\"utf-8\"?>");
      out("<!DOCTYPE html PUBLIC \"-//W3C//DTD XHTML 1.1//EN\"");
      out("\"http://www.w3.org/TR/xhtml1/DTD/xhtml11\">");
      out("<html xml:lang=\"en\" xmlns=\"http://www.w3.org/1999/xhtml\">");
      out("   <head>");
      out("      <title>"+socname+(title==""?"":": "+title)+"</title>");
      out("      <meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\" />");
      out("      <link rel=\"stylesheet\" type=\"text/css\" href=\""+basePath+"css/scotsman.css\" title=\"Scotsman\"/>");
      out("      <link rel=\"alternate stylesheet\" type=\"text/css\" href=\""+basePath+"css/assassins.css\" title=\"Original\"/>");
		printJavaScript();
      out("   </head>");
      out("   <body>");
		out("   <div id=\"header\">");
      out("      <img class=\"dagger\" src=\""+basePath+"img/cloak2.gif\" alt=\"Cloak &amp; Dagger\" /> ");
		out("      <div class=\"h1\">");
      out("         <hr />");
      out("		     <h1>");
      out("            "+socname+(title==""?"":": "+title));
      out("         </h1>");
      out("         <hr />");
		out("	     </div>");
      if (customnowtime) {
         out("   <h2>Acting at time: "+Tools.formatTime(Tools.getNowTime())+"</h2>");
         out("   <hr />");
      }
		out("   </div>");
		String s="";
		try {
			s = currentAuth.getEmail();
		} catch (AssassinSQLException asqle) {
			s="Error";
		}
		if (!"".equals(s)) {
			out("   <div id=\"auth\">");
			out("      <div class=\"status\">Logged In</div>");
			out("      <div class=\"username\">"+s+"</div>");
			out("      <div class=\"logout\">");
			CGITools.printlink(this,"logout","umpire.cgi?logout","Logout");
			out("      </div>");
			out("   </div>");
		}
		out("	<div id=\"container\">");
   }

   void sendHTMLFooter(boolean printNav) throws AssassinSQLException, CGIInvalidContentFormatException
   {
		if(printNav) {
			printLeftNav(this);
		}
      String email;
      try {
         email = Config.getConfig("vanilla", "from-address");
      } catch (NullPointerException NPex) { email = ""; }
		out("   </div>");		/* container */
		out("   <div class=\"navclear\"></div>");
		out("   <div id=\"footer\">");
      out("      <hr />   ");
      out("      <p class=\"center\">");
      out("         <a href=\"/assassins/\">Home</a> / <a href=\"mailto:"+email+"\">Email</a>");
      out("         <br />");
      out("         <a href=\"http://validator.w3.org/check/referer\"><img ");
      out("         src=\""+basePath+"img/valid-xhtml11.png\" alt=\"Valid XHTML 1.1\" /></a>");
      out("      </p>");
      out("      <hr />");
		out("   </div>");
      out("   </body>");
      out("</html>");
   }
}
