package org.ucam.srcf.assassins.xml;

import java.io.OutputStream;
import java.io.Writer;
import java.io.PrintWriter;
import java.util.Comparator;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.Stack;

import com.icl.saxon.aelfred.DefaultHandler;

import org.ucam.srcf.assassins.Assassin;
import org.ucam.srcf.assassins.AssassinState;
import org.ucam.srcf.assassins.Debug;
import org.ucam.srcf.assassins.Tools;
import org.xml.sax.Attributes;

public abstract class Handler extends DefaultHandler {

	protected PrintWriter out;
	protected String target;
	protected Set linkedPages;
	protected Map plugins;
	protected Map state;
	protected List currentList;
	protected int currentState;
	protected Map columns;
	protected List columnList;
	protected AssComp sorter;
	protected List filters;
	protected Stack pendingFilters;
	protected Set toRemove;
	protected String currentColourFilter;

	protected static int NOTHING = 0;
	protected static int TEXT    = 1;
	protected static int EMPTY   = 2;
	
	public Handler(OutputStream out,Map state,Map plugins) {
		this(out,"",state,plugins,new HashSet());
	}
		
	public Handler(Writer out,String target,Map state,Map plugins,Set pages) {
		this.out = new PrintWriter(out);
		this.target = target;
		this.state = state;
		this.plugins = plugins;
		this.linkedPages = pages;
	}

	public Handler(OutputStream out,String target,Map state,Map plugins,Set pages) {
		this.out = new PrintWriter(out);
		this.target = target;
		this.state = state;
		this.plugins = plugins;
		this.linkedPages = pages;
	}

	public void characters(char[] ch, int start, int length) {
		if (currentState == Handler.TEXT || (currentState==Handler.EMPTY && currentList.size()==0)) {
			String s = new String(ch, start, length);
			out.print(Tools.escapeXML(s));
		}
	}
	
	protected class AssComp implements Comparator {
		private String plugin;

		private String property;

		private Map myState;

		private boolean desc;

		private String num;

		public AssComp(String pl, String pr, Map s, boolean desc, String num) {
			plugin = pl;
			property = pr;
			myState = s;
			this.desc = desc;
			this.num = num;
		}

		public int compare(Object o1, Object o2) {
			Assassin a1 = (Assassin) o1;
			Assassin a2 = (Assassin) o2;
			String s1 = (String) ((AssassinState) myState.get(a1)).getProperty(
					plugin, property);
			String s2 = (String) ((AssassinState) myState.get(a2)).getProperty(
					plugin, property);

			int i;
			if (null == s1 && null == s2)
				i = 0;
			else if (null == s1)
				i = -1;
			else if (null == s2)
				i = 1;
			else if (null != num) {
				// convert to Integer for numeric sort
				if (num.equalsIgnoreCase("INTEGER")) {
					int i1 = Integer.parseInt(s1);
					int i2 = Integer.parseInt(s2);
					i = i1 - i2;
				} else if (num.equalsIgnoreCase("DOUBLE")) {
					double d1 = Double.parseDouble(s1);
					double d2 = Double.parseDouble(s2);
					i = (d1<d2)?-1:((d1>d2)?1:0);
				} else i = s1.compareTo(s2);
			} else i = s1.compareTo(s2);

			// invert descending
			if (desc) i = -i;
			if (0 != i) return i;

			// compare hashcodes if equal
			int diff = a2.hashCode() - a1.hashCode();

			// invert descending
			if (desc)
				return -diff;
			else return diff;
		}

		public boolean equals(Object obj) {
			return (compare(this, obj) == 0);
		}
	}

	public void startDocument() {}
	public void endDocument() {}
	
	public void startElement(String namespaceURI, String localName, String qName, Attributes atts) {}
	public void endElement(String namespaceURI, String localName, String qName) {}
	
	
}
