package org.ucam.srcf.assassins.xml;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.PrintWriter;
import java.util.ArrayList;
import java.util.Comparator;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.TreeSet;

import org.ucam.srcf.assassins.Assassin;
import org.ucam.srcf.assassins.AssassinState;
import org.ucam.srcf.assassins.AssassinStateViewer;
import org.ucam.srcf.assassins.Debug;
import org.ucam.srcf.assassins.NewsPage;
import org.ucam.srcf.assassins.Plugin;
import org.ucam.srcf.assassins.exceptions.AssassinSQLException;
import org.ucam.srcf.assassins.exceptions.XMLParseException;
import org.ucam.srcf.assassins.Tools;

import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

public class WebPageParserDOM extends WebPageParser {

	protected XMLParser myXMLParser;
	
	public WebPageParserDOM (String filename, Map state, Map plugins, String target, Set pages) throws AssassinSQLException, XMLParseException {
      if (Debug.debug) Debug.print(WebPageParserDOM.class, "Constructor called");
      myXMLParser = new XMLParser(filename,false,false,false,false,false);
      myState=state;
      myPlugins=plugins;
      try {
         out = new PrintWriter(new FileOutputStream(new File(target)));
         this.target=target;
         this.linkedPages=pages;
      } catch (IOException e) {
         if (Debug.debug) Debug.print(WebPageParserDOM.class, "Here");
         e.printStackTrace();
      }
      if (Debug.debug) Debug.print(WebPageParserDOM.class, "Constructor finished");
   }
   
   public void outputContentsOfNode(Node y) {
      NodeList nl3 = y.getChildNodes();
      for (int c=0; c<nl3.getLength(); c++) {
         Node y3 = nl3.item(c);
         if (y3.getNodeName().equals("#text")) {
            out.print(y3.getNodeValue());
         } else {
            out.print("<"+y3.getNodeName()+">");
            outputContentsOfNode(y3);
            out.print("</"+y3.getNodeName()+">");
         }
      }
   }


   /* (non-Javadoc)
 * @see org.ucam.srcf.assassins.xml.WebPageParser#generate()
 */
public void generate() throws AssassinSQLException {
      if (Debug.debug) Debug.print(WebPageParserDOM.class, "Generate called");
      out.println("<?xml version=\"1.0\" encoding=\"UTF-8\" ?>");
      out.println("<!DOCTYPE webpage SYSTEM \"ListPage.dtd\" >");
      out.println("<webpage>");
      NodeList nl = myXMLParser.getTagList("webpage");
      Node x = nl.item(0);
      NodeList nl2 = x.getChildNodes();
      for (int b=0; b<nl2.getLength(); b++) {
         Node y = nl2.item(b);
         if (y.getNodeName().equals("title")) {
            out.print("  <title>");
            outputContentsOfNode(y);
            out.println("  </title>");
         }
         if (y.getNodeName().equals("text")) {
            out.print("  <text>");
            outputContentsOfNode(y);
            out.println("  </text>");
         }
         if (y.getNodeName().equals("list")) {
            parselist(y);
         }
      }
      out.println("  <links>");
      for (Iterator i = linkedPages.iterator(); i.hasNext();) {
      	NewsPage np = (NewsPage)i.next();
      	String l = np.getLinkName();
      	String t = np.getTitle();
      	if (l!=target) out.println("   <a href=\""+l+"\">"+t+"</a>");
      }
      out.println("    <a href=\""+Config.getConfig("vanilla","updates-page-name")+".html\">Updates</a>");
      out.println("    <a href=\""+Config.getConfig("vanilla","headlines-page-name")+".html\">Headlines</a>");
      int index=1;
      String s=null;
      while (null!=(s=Config.getConfig("vanilla","template-page-name-"+index))) {
      	out.println("    <a href=\""+s+".html\">"+s+"</a>");
         index++;
      }
      out.println("  </links>");
      out.println("</webpage>");
      out.close();
   }
   private class AssComp implements Comparator {
	   private String plugin;
	   private String property;
	   private Map myState;
      private boolean desc;
      private boolean num;
	   public AssComp(String pl, String pr, Map s, boolean desc, boolean num) {
		   plugin = pl;
		   property = pr;
		   myState = s;
         this.desc = desc;
         this.num = num;
	   }
	   public int compare(Object o1, Object o2) {
		   Assassin a1 = (Assassin) o1;
		   Assassin a2 = (Assassin) o2;
		   String s1 = (String) ((AssassinState)myState.get(a1)).getProperty(plugin,property);
		   String s2 = (String) ((AssassinState)myState.get(a2)).getProperty(plugin,property);

         int i;
         if (null == s1 && null == s2)
            i = 0;
         else if (null == s1)
            i = -1;
         else if (null == s2)
            i = 1;
         else {
            // convert to Integer for numeric sort
            if (num) {
               int i1 = Integer.parseInt(s1);
               int i2 = Integer.parseInt(s2);
               i = i1-i2;
            }
            else
               i = s1.compareTo(s2);
         }
         
         // invert descending
         if (desc) i = -i;
		   if (0 != i) return i;

         // compare hashcodes if equal
		   int diff = a2.hashCode() - a1.hashCode();

         // invert descending
         if (desc) return -diff;
         else return diff;
	   }
	   public boolean equals(Object obj) {
		   return (compare(this,obj) == 0);
	   }
   }
	   
   private void parselist(Node y) throws AssassinSQLException {
      if (Debug.debug) Debug.print(this,"Starting List Handler");
      HashMap zz = new HashMap(myState);
      Set z=zz.keySet();
      Iterator i = z.iterator();
      List q = new ArrayList();
      while (i.hasNext()) {
         Object o = i.next();
         Assassin a = (Assassin)(o);
         if (Debug.debug) Debug.print(this, "Assassin "+a);
         NodeList nl2 = myXMLParser.getTagListFromNode(y,"and");
         boolean include = true;
         for (int x=0; null!=nl2 && x<nl2.getLength(); x++) {
            // There can be only one (hopefully) but if not...
            include&=parseAnd(a,nl2.item(x));
         }
         nl2 = myXMLParser.getTagListFromNode(y,"or");
         if (null!=nl2 && nl2.getLength() > 0) include=false;
         for (int x=0; null!=nl2 && x<nl2.getLength(); x++) {
            // There can be only one (hopefully) but if not...
            include|=parseOr(a,nl2.item(x));
         }
         if (include) q.add(a);
         if (Debug.debug) Debug.print(this,"Done Assassin "+a+", include = " + include);
      }
      if (Debug.debug) Debug.print(this, "Done All Assassins!");
      // Now we have a Set of Assassins that we can use...
      if (q.isEmpty()) {
         if (Debug.debug) Debug.print(this,"Empty");
         NodeList nl = myXMLParser.getTagListFromNode(y,"empty");
         if (Debug.debug) Debug.print(this,"NL:"+nl);
         Element x = (Element) nl.item(0);
         if (null==x) {
            // Do nothing
         } else {
            out.println("  <text>");
            outputContentsOfNode(x);
            out.println("  </text>");
         }
      } else {
         out.println("  <table class=\"playerlist\">");
	 NodeList sl = myXMLParser.getTagListFromNode(y,"sort");
	 Set sortset;
	 if (sl.getLength() > 0) {
		 Element sortel = (Element) sl.item(0);
		 String sortpl = sortel.getAttribute("plugin");
		 String sortpr = sortel.getAttribute("name");
       boolean desc = false;
       boolean num = false;
		 if ("true".equals(sortel.getAttribute("ascending"))) desc = false;
       else if ("true".equals(sortel.getAttribute("descending"))) desc = true;
       else if ("false".equals(sortel.getAttribute("ascending"))) desc = true;
       else if ("false".equals(sortel.getAttribute("descending"))) desc = false;
		 if ("true".equals(sortel.getAttribute("numeric"))) num = true;
       if (Debug.debug) Debug.print(WebPageParserDOM.class, " asc: "+sortel.getAttribute("ascending")+
             " desc: "+sortel.getAttribute("descending")+" num: "+sortel.getAttribute("numeric")+" d/n: "+desc+"/"+num);
	 	 sortset = new TreeSet(new AssComp(sortpl, sortpr, myState, desc, num));
	 } else sortset = new HashSet();
	 sortset.addAll(q);
         NodeList nl = myXMLParser.getTagListFromNode(y,"column");
         out.println("    <tr>");
         for (int b=0; b<nl.getLength(); b++) {
            Element x = (Element) nl.item(b);
            out.println("      <td>"+x.getAttribute("title")+"</td>");
         }
         out.println("    </tr>");
         if (Debug.debug) Debug.print(this,"Done headings");
         Iterator i2 = sortset.iterator();
         while (i2.hasNext()) {
            Assassin a = (Assassin) (i2.next());
            if (Debug.debug) Debug.print(this,"Assassin: "+a+": myState(a): "+myState.get(a));
            out.println("    <tr>");
            for (int x=0; x<nl.getLength(); x++) {
               Element zb = (Element) nl.item(x);
               String name = zb.getAttribute("name");
               String plugin = zb.getAttribute("plugin");
               if (null==plugin || "".equals(plugin)) plugin="vanilla";
               String x2 = (String) ((AssassinState)myState.get(a)).getProperty(plugin,name);
               out.println("      <td>"+Tools.obfuscateEmail(Tools.escapeXML(x2))+"</td>");
            }
            out.println("    </tr>");
         }
         out.println("  </table>");
         out.flush();
      }
   }

   private boolean parseAnd(Assassin a, Node z) throws AssassinSQLException {
      NodeList or = myXMLParser.getTagListFromNode(z,"or");
      boolean i=true;
      if (null!=or && or.getLength() > 0) i = false;
      for (int x=0; null!=or && x<or.getLength(); x++) {
         if (Debug.debug) Debug.print(this, "looping pointlessly");
	 Node y = or.item(x);
         i|=parseOr(a,y);
      }
      if (Debug.debug) Debug.print(this, "Checked Internal OR, i = "+ i);
      NodeList nl = myXMLParser.getTagListFromNode(z,"filter");
      for (int x=0; x<nl.getLength(); x++) {
         Element y = (Element) nl.item(x);
         String plugin = y.getAttribute("plugin");
         if (null==plugin || "".equals(plugin)) plugin="vanilla";
         if (Debug.debug) Debug.print(this, "Plugin: "+plugin);
         Plugin pl = (Plugin) myPlugins.get(plugin);
         boolean b = pl.assassinHasProperty(new AssassinStateViewer((AssassinState)myState.get(a)), y.getAttribute("property"),y.getAttribute("condition"));
	 i &= b;
	 if (Debug.debug) Debug.print(this,"AND: Checked "+pl+" for "+y.getAttribute("condition")+"="+y.getAttribute("property") + "Got: " + b);
      }
      return i;
   }

   private boolean parseOr(Assassin a, Node z) throws AssassinSQLException {
      NodeList and = myXMLParser.getTagListFromNode(z,"and");
      boolean i = false;
      if (null!=and) i = true;
      for (int x=0; null!=and && x<and.getLength(); x++) {
         Node y = and.item(x);
         i&=parseAnd(a,y);
      }
      NodeList nl = myXMLParser.getTagListFromNode(z,"filter");
      for (int x=0; x<nl.getLength(); x++) {
         Element y = (Element) nl.item(x);
         String plugin = y.getAttribute("plugin");
         if (null==plugin) plugin="vanilla";
         Plugin pl = (Plugin) myPlugins.get(plugin);
         i |= pl.assassinHasProperty(new AssassinStateViewer((AssassinState)myState.get(a)), y.getAttribute("property"),y.getAttribute("condition"));
         if (Debug.debug) Debug.print(this,"OR: Checked "+pl+" for "+y.getAttribute("condition")+"="+y.getAttribute("property"));
      }
      return i;
   }
}  
